# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

LABEL_DILATE_METADATA = Metadata(
    id="e02283fd8307f04426742148eee8e3e920bc3fac.workbench",
    name="label-dilate",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_LabelDilateParamsDictNoTag = typing.TypedDict('_LabelDilateParamsDictNoTag', {
    "label": InputPathType,
    "surface": InputPathType,
    "dilate-dist": float,
    "label-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
LabelDilateParamsDictTagged = typing.TypedDict('LabelDilateParamsDictTagged', {
    "@type": typing.Literal["workbench/label-dilate"],
    "label": InputPathType,
    "surface": InputPathType,
    "dilate-dist": float,
    "label-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
LabelDilateParamsDict = _LabelDilateParamsDictNoTag | LabelDilateParamsDictTagged


class LabelDilateOutputs(typing.NamedTuple):
    """
    Output object returned when calling `LabelDilateParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    label_out: OutputPathType
    """the output label file"""


def label_dilate_params(
    label: InputPathType,
    surface: InputPathType,
    dilate_dist: float,
    label_out: str,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
) -> LabelDilateParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label: the input label.
        surface: the surface to dilate on.
        dilate_dist: distance in mm to dilate the labels.
        label_out: the output label file.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column to dilate\
            \
            the column number or name.
        roi_metric: specify an roi of vertices to overwrite, rather than\
            vertices with the unlabeled key\
            \
            metric file, positive values denote vertices to have their values\
            replaced.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/label-dilate",
        "label": label,
        "surface": surface,
        "dilate-dist": dilate_dist,
        "label-out": label_out,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def label_dilate_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `LabelDilateParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label", None) is None:
        raise StyxValidationError("`label` must not be None")
    if not isinstance(params["label"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label` has the wrong type: Received `{type(params.get("label", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("dilate-dist", None) is None:
        raise StyxValidationError("`dilate-dist` must not be None")
    if not isinstance(params["dilate-dist"], (float, int)):
        raise StyxValidationError(f'`dilate-dist` has the wrong type: Received `{type(params.get("dilate-dist", None))}` expected `float`')
    if params.get("label-out", None) is None:
        raise StyxValidationError("`label-out` must not be None")
    if not isinstance(params["label-out"], str):
        raise StyxValidationError(f'`label-out` has the wrong type: Received `{type(params.get("label-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def label_dilate_cargs(
    params: LabelDilateParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-label-dilate"
    ])
    cargs.append(execution.input_file(params.get("label", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("dilate-dist", None)))
    cargs.append(params.get("label-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-bad-vertex-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def label_dilate_outputs(
    params: LabelDilateParamsDict,
    execution: Execution,
) -> LabelDilateOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = LabelDilateOutputs(
        root=execution.output_file("."),
        label_out=execution.output_file(params.get("label-out", None)),
    )
    return ret


def label_dilate_execute(
    params: LabelDilateParamsDict,
    runner: Runner | None = None,
) -> LabelDilateOutputs:
    """
    DILATE A LABEL FILE.
    
    Fills in label information for all vertices designated as bad, up to the
    specified distance away from other labels. If -bad-vertex-roi is specified,
    all vertices, including those with the unlabeled key, are good, except for
    vertices with a positive value in the ROI. If it is not specified, only
    vertices with the unlabeled key are bad.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelDilateOutputs`).
    """
    label_dilate_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(LABEL_DILATE_METADATA)
    params = execution.params(params)
    cargs = label_dilate_cargs(params, execution)
    ret = label_dilate_outputs(params, execution)
    execution.run(cargs)
    return ret


def label_dilate(
    label: InputPathType,
    surface: InputPathType,
    dilate_dist: float,
    label_out: str,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> LabelDilateOutputs:
    """
    DILATE A LABEL FILE.
    
    Fills in label information for all vertices designated as bad, up to the
    specified distance away from other labels. If -bad-vertex-roi is specified,
    all vertices, including those with the unlabeled key, are good, except for
    vertices with a positive value in the ROI. If it is not specified, only
    vertices with the unlabeled key are bad.
    
    Args:
        label: the input label.
        surface: the surface to dilate on.
        dilate_dist: distance in mm to dilate the labels.
        label_out: the output label file.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column to dilate\
            \
            the column number or name.
        roi_metric: specify an roi of vertices to overwrite, rather than\
            vertices with the unlabeled key\
            \
            metric file, positive values denote vertices to have their values\
            replaced.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `LabelDilateOutputs`).
    """
    params = label_dilate_params(
        label=label,
        surface=surface,
        dilate_dist=dilate_dist,
        label_out=label_out,
        area_metric=area_metric,
        column=column,
        roi_metric=roi_metric,
    )
    return label_dilate_execute(params, runner)


__all__ = [
    "LABEL_DILATE_METADATA",
    "LabelDilateOutputs",
    "LabelDilateParamsDict",
    "LabelDilateParamsDictTagged",
    "label_dilate",
    "label_dilate_execute",
    "label_dilate_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_ERODE_METADATA = Metadata(
    id="778174a7101c157fd44e9efd68314bd69963400f.workbench",
    name="metric-erode",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricErodeParamsDictNoTag = typing.TypedDict('_MetricErodeParamsDictNoTag', {
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricErodeParamsDictTagged = typing.TypedDict('MetricErodeParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-erode"],
    "metric": InputPathType,
    "surface": InputPathType,
    "distance": float,
    "metric-out": str,
    "area-metric": typing.NotRequired[InputPathType | None],
    "column": typing.NotRequired[str | None],
    "roi-metric": typing.NotRequired[InputPathType | None],
})
MetricErodeParamsDict = _MetricErodeParamsDictNoTag | MetricErodeParamsDictTagged


class MetricErodeOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricErodeParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_erode_params(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
) -> MetricErodeParamsDictTagged:
    """
    Build parameters.
    
    Args:
        metric: the metric file to erode.
        surface: the surface to compute on.
        distance: distance in mm to erode.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column to erode\
            \
            the column number or name.
        roi_metric: assume values outside this roi are nonzero\
            \
            metric file, positive values denote vertices that have data.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-erode",
        "metric": metric,
        "surface": surface,
        "distance": distance,
        "metric-out": metric_out,
    }
    if area_metric is not None:
        params["area-metric"] = area_metric
    if column is not None:
        params["column"] = column
    if roi_metric is not None:
        params["roi-metric"] = roi_metric
    return params


def metric_erode_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricErodeParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("distance", None) is None:
        raise StyxValidationError("`distance` must not be None")
    if not isinstance(params["distance"], (float, int)):
        raise StyxValidationError(f'`distance` has the wrong type: Received `{type(params.get("distance", None))}` expected `float`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("area-metric", None) is not None:
        if not isinstance(params["area-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`area-metric` has the wrong type: Received `{type(params.get("area-metric", None))}` expected `InputPathType | None`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("roi-metric", None) is not None:
        if not isinstance(params["roi-metric"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-metric` has the wrong type: Received `{type(params.get("roi-metric", None))}` expected `InputPathType | None`')


def metric_erode_cargs(
    params: MetricErodeParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-erode"
    ])
    cargs.append(execution.input_file(params.get("metric", None)))
    cargs.append(execution.input_file(params.get("surface", None)))
    cargs.append(str(params.get("distance", None)))
    cargs.append(params.get("metric-out", None))
    if params.get("area-metric", None) is not None:
        cargs.extend([
            "-corrected-areas",
            execution.input_file(params.get("area-metric", None))
        ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("roi-metric", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-metric", None))
        ])
    return cargs


def metric_erode_outputs(
    params: MetricErodeParamsDict,
    execution: Execution,
) -> MetricErodeOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricErodeOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_erode_execute(
    params: MetricErodeParamsDict,
    runner: Runner | None = None,
) -> MetricErodeOutputs:
    """
    ERODE A METRIC FILE.
    
    Around each vertex with a value of zero, set surrounding vertices to zero.
    The surrounding vertices are all immediate neighbors and all vertices within
    the specified distance.
    
    Note that the -corrected-areas option uses an approximate correction for
    distance along the surface.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricErodeOutputs`).
    """
    metric_erode_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_ERODE_METADATA)
    params = execution.params(params)
    cargs = metric_erode_cargs(params, execution)
    ret = metric_erode_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_erode(
    metric: InputPathType,
    surface: InputPathType,
    distance: float,
    metric_out: str,
    area_metric: InputPathType | None = None,
    column: str | None = None,
    roi_metric: InputPathType | None = None,
    runner: Runner | None = None,
) -> MetricErodeOutputs:
    """
    ERODE A METRIC FILE.
    
    Around each vertex with a value of zero, set surrounding vertices to zero.
    The surrounding vertices are all immediate neighbors and all vertices within
    the specified distance.
    
    Note that the -corrected-areas option uses an approximate correction for
    distance along the surface.
    
    Args:
        metric: the metric file to erode.
        surface: the surface to compute on.
        distance: distance in mm to erode.
        metric_out: the output metric.
        area_metric: vertex areas to use instead of computing them from the\
            surface\
            \
            the corrected vertex areas, as a metric.
        column: select a single column to erode\
            \
            the column number or name.
        roi_metric: assume values outside this roi are nonzero\
            \
            metric file, positive values denote vertices that have data.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricErodeOutputs`).
    """
    params = metric_erode_params(
        metric=metric,
        surface=surface,
        distance=distance,
        metric_out=metric_out,
        area_metric=area_metric,
        column=column,
        roi_metric=roi_metric,
    )
    return metric_erode_execute(params, runner)


__all__ = [
    "METRIC_ERODE_METADATA",
    "MetricErodeOutputs",
    "MetricErodeParamsDict",
    "MetricErodeParamsDictTagged",
    "metric_erode",
    "metric_erode_execute",
    "metric_erode_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CREATE_SCALAR_SERIES_METADATA = Metadata(
    id="8d713a958679f48506f74cec988ea68de2166f03.workbench",
    name="cifti-create-scalar-series",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiCreateScalarSeriesSeriesParamsDictNoTag = typing.TypedDict('_CiftiCreateScalarSeriesSeriesParamsDictNoTag', {
    "unit": str,
    "start": float,
    "step": float,
})
CiftiCreateScalarSeriesSeriesParamsDictTagged = typing.TypedDict('CiftiCreateScalarSeriesSeriesParamsDictTagged', {
    "@type": typing.Literal["series"],
    "unit": str,
    "start": float,
    "step": float,
})
CiftiCreateScalarSeriesSeriesParamsDict = _CiftiCreateScalarSeriesSeriesParamsDictNoTag | CiftiCreateScalarSeriesSeriesParamsDictTagged


_CiftiCreateScalarSeriesParamsDictNoTag = typing.TypedDict('_CiftiCreateScalarSeriesParamsDictNoTag', {
    "input": str,
    "cifti-out": str,
    "series": typing.NotRequired[CiftiCreateScalarSeriesSeriesParamsDict | None],
    "file": typing.NotRequired[str | None],
    "transpose": bool,
})
CiftiCreateScalarSeriesParamsDictTagged = typing.TypedDict('CiftiCreateScalarSeriesParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-create-scalar-series"],
    "input": str,
    "cifti-out": str,
    "series": typing.NotRequired[CiftiCreateScalarSeriesSeriesParamsDict | None],
    "file": typing.NotRequired[str | None],
    "transpose": bool,
})
CiftiCreateScalarSeriesParamsDict = _CiftiCreateScalarSeriesParamsDictNoTag | CiftiCreateScalarSeriesParamsDictTagged


def cifti_create_scalar_series_series(
    unit: str,
    start: float,
    step: float,
) -> CiftiCreateScalarSeriesSeriesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        unit: the unit to use.
        start: the value at the first series point.
        step: the interval between series points.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "series",
        "unit": unit,
        "start": start,
        "step": step,
    }
    return params


def cifti_create_scalar_series_series_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateScalarSeriesSeriesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("unit", None) is None:
        raise StyxValidationError("`unit` must not be None")
    if not isinstance(params["unit"], str):
        raise StyxValidationError(f'`unit` has the wrong type: Received `{type(params.get("unit", None))}` expected `str`')
    if params.get("start", None) is None:
        raise StyxValidationError("`start` must not be None")
    if not isinstance(params["start"], (float, int)):
        raise StyxValidationError(f'`start` has the wrong type: Received `{type(params.get("start", None))}` expected `float`')
    if params.get("step", None) is None:
        raise StyxValidationError("`step` must not be None")
    if not isinstance(params["step"], (float, int)):
        raise StyxValidationError(f'`step` has the wrong type: Received `{type(params.get("step", None))}` expected `float`')


def cifti_create_scalar_series_series_cargs(
    params: CiftiCreateScalarSeriesSeriesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-series",
        params.get("unit", None),
        str(params.get("start", None)),
        str(params.get("step", None))
    ])
    return cargs


class CiftiCreateScalarSeriesOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiCreateScalarSeriesParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """output cifti file"""


def cifti_create_scalar_series_params(
    input_: str,
    cifti_out: str,
    series: CiftiCreateScalarSeriesSeriesParamsDict | None = None,
    file: str | None = None,
    transpose: bool = False,
) -> CiftiCreateScalarSeriesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_: input file.
        cifti_out: output cifti file.
        series: set the units and values of the series.
        file: use a text file to set names on scalar dimension\
            \
            text file containing names, one per line.
        transpose: use if the rows of the text file are along the scalar\
            dimension.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-create-scalar-series",
        "input": input_,
        "cifti-out": cifti_out,
        "transpose": transpose,
    }
    if series is not None:
        params["series"] = series
    if file is not None:
        params["file"] = file
    return params


def cifti_create_scalar_series_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiCreateScalarSeriesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input", None) is None:
        raise StyxValidationError("`input` must not be None")
    if not isinstance(params["input"], str):
        raise StyxValidationError(f'`input` has the wrong type: Received `{type(params.get("input", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("series", None) is not None:
        cifti_create_scalar_series_series_validate(params["series"])
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')
    if params.get("transpose", False) is None:
        raise StyxValidationError("`transpose` must not be None")
    if not isinstance(params["transpose"], bool):
        raise StyxValidationError(f'`transpose` has the wrong type: Received `{type(params.get("transpose", False))}` expected `bool`')


def cifti_create_scalar_series_cargs(
    params: CiftiCreateScalarSeriesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-create-scalar-series"
    ])
    cargs.append(params.get("input", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("series", None) is not None:
        cargs.extend(cifti_create_scalar_series_series_cargs(params.get("series", None), execution))
    if params.get("file", None) is not None:
        cargs.extend([
            "-name-file",
            params.get("file", None)
        ])
    if params.get("transpose", False):
        cargs.append("-transpose")
    return cargs


def cifti_create_scalar_series_outputs(
    params: CiftiCreateScalarSeriesParamsDict,
    execution: Execution,
) -> CiftiCreateScalarSeriesOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiCreateScalarSeriesOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_create_scalar_series_execute(
    params: CiftiCreateScalarSeriesParamsDict,
    runner: Runner | None = None,
) -> CiftiCreateScalarSeriesOutputs:
    """
    IMPORT SERIES DATA INTO CIFTI.
    
    Convert a text file containing series of equal length into a cifti file. The
    text file should have lines made up of numbers separated by whitespace, with
    no extra newlines between lines.
    
    The <unit> argument must be one of the following:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateScalarSeriesOutputs`).
    """
    cifti_create_scalar_series_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CREATE_SCALAR_SERIES_METADATA)
    params = execution.params(params)
    cargs = cifti_create_scalar_series_cargs(params, execution)
    ret = cifti_create_scalar_series_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_create_scalar_series(
    input_: str,
    cifti_out: str,
    series: CiftiCreateScalarSeriesSeriesParamsDict | None = None,
    file: str | None = None,
    transpose: bool = False,
    runner: Runner | None = None,
) -> CiftiCreateScalarSeriesOutputs:
    """
    IMPORT SERIES DATA INTO CIFTI.
    
    Convert a text file containing series of equal length into a cifti file. The
    text file should have lines made up of numbers separated by whitespace, with
    no extra newlines between lines.
    
    The <unit> argument must be one of the following:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        input_: input file.
        cifti_out: output cifti file.
        series: set the units and values of the series.
        file: use a text file to set names on scalar dimension\
            \
            text file containing names, one per line.
        transpose: use if the rows of the text file are along the scalar\
            dimension.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiCreateScalarSeriesOutputs`).
    """
    params = cifti_create_scalar_series_params(
        input_=input_,
        cifti_out=cifti_out,
        series=series,
        file=file,
        transpose=transpose,
    )
    return cifti_create_scalar_series_execute(params, runner)


__all__ = [
    "CIFTI_CREATE_SCALAR_SERIES_METADATA",
    "CiftiCreateScalarSeriesOutputs",
    "CiftiCreateScalarSeriesParamsDict",
    "CiftiCreateScalarSeriesParamsDictTagged",
    "CiftiCreateScalarSeriesSeriesParamsDict",
    "CiftiCreateScalarSeriesSeriesParamsDictTagged",
    "cifti_create_scalar_series",
    "cifti_create_scalar_series_execute",
    "cifti_create_scalar_series_params",
    "cifti_create_scalar_series_series",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA = Metadata(
    id="89e83249d5059a7e652b2c0a10e95f7e3d72df57.workbench",
    name="volume-warpfield-affine-regression",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeWarpfieldAffineRegressionFlirtOutParamsDictNoTag = typing.TypedDict('_VolumeWarpfieldAffineRegressionFlirtOutParamsDictNoTag', {
    "source-volume": str,
    "target-volume": str,
})
VolumeWarpfieldAffineRegressionFlirtOutParamsDictTagged = typing.TypedDict('VolumeWarpfieldAffineRegressionFlirtOutParamsDictTagged', {
    "@type": typing.Literal["flirt-out"],
    "source-volume": str,
    "target-volume": str,
})
VolumeWarpfieldAffineRegressionFlirtOutParamsDict = _VolumeWarpfieldAffineRegressionFlirtOutParamsDictNoTag | VolumeWarpfieldAffineRegressionFlirtOutParamsDictTagged


_VolumeWarpfieldAffineRegressionParamsDictNoTag = typing.TypedDict('_VolumeWarpfieldAffineRegressionParamsDictNoTag', {
    "warpfield": str,
    "affine-out": str,
    "flirt-out": typing.NotRequired[VolumeWarpfieldAffineRegressionFlirtOutParamsDict | None],
    "source-volume": typing.NotRequired[str | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
})
VolumeWarpfieldAffineRegressionParamsDictTagged = typing.TypedDict('VolumeWarpfieldAffineRegressionParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-warpfield-affine-regression"],
    "warpfield": str,
    "affine-out": str,
    "flirt-out": typing.NotRequired[VolumeWarpfieldAffineRegressionFlirtOutParamsDict | None],
    "source-volume": typing.NotRequired[str | None],
    "roi-vol": typing.NotRequired[InputPathType | None],
})
VolumeWarpfieldAffineRegressionParamsDict = _VolumeWarpfieldAffineRegressionParamsDictNoTag | VolumeWarpfieldAffineRegressionParamsDictTagged


def volume_warpfield_affine_regression_flirt_out(
    source_volume: str,
    target_volume: str,
) -> VolumeWarpfieldAffineRegressionFlirtOutParamsDictTagged:
    """
    Build parameters.
    
    Args:
        source_volume: the volume you want to apply the transform to.
        target_volume: the target space you want the transformed volume to\
            match.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "flirt-out",
        "source-volume": source_volume,
        "target-volume": target_volume,
    }
    return params


def volume_warpfield_affine_regression_flirt_out_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeWarpfieldAffineRegressionFlirtOutParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("source-volume", None) is None:
        raise StyxValidationError("`source-volume` must not be None")
    if not isinstance(params["source-volume"], str):
        raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str`')
    if params.get("target-volume", None) is None:
        raise StyxValidationError("`target-volume` must not be None")
    if not isinstance(params["target-volume"], str):
        raise StyxValidationError(f'`target-volume` has the wrong type: Received `{type(params.get("target-volume", None))}` expected `str`')


def volume_warpfield_affine_regression_flirt_out_cargs(
    params: VolumeWarpfieldAffineRegressionFlirtOutParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-flirt-out",
        params.get("source-volume", None),
        params.get("target-volume", None)
    ])
    return cargs


class VolumeWarpfieldAffineRegressionOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeWarpfieldAffineRegressionParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_warpfield_affine_regression_params(
    warpfield: str,
    affine_out: str,
    flirt_out: VolumeWarpfieldAffineRegressionFlirtOutParamsDict | None = None,
    source_volume: str | None = None,
    roi_vol: InputPathType | None = None,
) -> VolumeWarpfieldAffineRegressionParamsDictTagged:
    """
    Build parameters.
    
    Args:
        warpfield: the input warpfield.
        affine_out: output - the output affine file.
        flirt_out: write output as a flirt matrix rather than a world\
            coordinate transform.
        source_volume: input is a fnirt warpfield\
            \
            the source volume used when generating the fnirt warpfield.
        roi_vol: only consider voxels within a mask (e.g., a brain mask)\
            \
            the mask volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-warpfield-affine-regression",
        "warpfield": warpfield,
        "affine-out": affine_out,
    }
    if flirt_out is not None:
        params["flirt-out"] = flirt_out
    if source_volume is not None:
        params["source-volume"] = source_volume
    if roi_vol is not None:
        params["roi-vol"] = roi_vol
    return params


def volume_warpfield_affine_regression_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeWarpfieldAffineRegressionParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("warpfield", None) is None:
        raise StyxValidationError("`warpfield` must not be None")
    if not isinstance(params["warpfield"], str):
        raise StyxValidationError(f'`warpfield` has the wrong type: Received `{type(params.get("warpfield", None))}` expected `str`')
    if params.get("affine-out", None) is None:
        raise StyxValidationError("`affine-out` must not be None")
    if not isinstance(params["affine-out"], str):
        raise StyxValidationError(f'`affine-out` has the wrong type: Received `{type(params.get("affine-out", None))}` expected `str`')
    if params.get("flirt-out", None) is not None:
        volume_warpfield_affine_regression_flirt_out_validate(params["flirt-out"])
    if params.get("source-volume", None) is not None:
        if not isinstance(params["source-volume"], str):
            raise StyxValidationError(f'`source-volume` has the wrong type: Received `{type(params.get("source-volume", None))}` expected `str | None`')
    if params.get("roi-vol", None) is not None:
        if not isinstance(params["roi-vol"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-vol` has the wrong type: Received `{type(params.get("roi-vol", None))}` expected `InputPathType | None`')


def volume_warpfield_affine_regression_cargs(
    params: VolumeWarpfieldAffineRegressionParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-warpfield-affine-regression"
    ])
    cargs.append(params.get("warpfield", None))
    cargs.append(params.get("affine-out", None))
    if params.get("flirt-out", None) is not None:
        cargs.extend(volume_warpfield_affine_regression_flirt_out_cargs(params.get("flirt-out", None), execution))
    if params.get("source-volume", None) is not None:
        cargs.extend([
            "-fnirt",
            params.get("source-volume", None)
        ])
    if params.get("roi-vol", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-vol", None))
        ])
    return cargs


def volume_warpfield_affine_regression_outputs(
    params: VolumeWarpfieldAffineRegressionParamsDict,
    execution: Execution,
) -> VolumeWarpfieldAffineRegressionOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeWarpfieldAffineRegressionOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_warpfield_affine_regression_execute(
    params: VolumeWarpfieldAffineRegressionParamsDict,
    runner: Runner | None = None,
) -> VolumeWarpfieldAffineRegressionOutputs:
    """
    REGRESS AFFINE FROM WARPFIELD.
    
    For all voxels in the warpfield, do a regression that predicts the post-warp
    coordinate from the source coordinate. When -roi is specified, only consider
    voxels with a value greater than 0 in <roi-vol>.
    
    The default is to expect the warpfield to be in relative world coordinates
    (mm space), and to write the output as a world affine (mm space to mm
    space). If you are using FSL-created files and utilities, specify -fnirt and
    -flirt-out as needed, as their coordinate conventions are different.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWarpfieldAffineRegressionOutputs`).
    """
    volume_warpfield_affine_regression_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA)
    params = execution.params(params)
    cargs = volume_warpfield_affine_regression_cargs(params, execution)
    ret = volume_warpfield_affine_regression_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_warpfield_affine_regression(
    warpfield: str,
    affine_out: str,
    flirt_out: VolumeWarpfieldAffineRegressionFlirtOutParamsDict | None = None,
    source_volume: str | None = None,
    roi_vol: InputPathType | None = None,
    runner: Runner | None = None,
) -> VolumeWarpfieldAffineRegressionOutputs:
    """
    REGRESS AFFINE FROM WARPFIELD.
    
    For all voxels in the warpfield, do a regression that predicts the post-warp
    coordinate from the source coordinate. When -roi is specified, only consider
    voxels with a value greater than 0 in <roi-vol>.
    
    The default is to expect the warpfield to be in relative world coordinates
    (mm space), and to write the output as a world affine (mm space to mm
    space). If you are using FSL-created files and utilities, specify -fnirt and
    -flirt-out as needed, as their coordinate conventions are different.
    
    Args:
        warpfield: the input warpfield.
        affine_out: output - the output affine file.
        flirt_out: write output as a flirt matrix rather than a world\
            coordinate transform.
        source_volume: input is a fnirt warpfield\
            \
            the source volume used when generating the fnirt warpfield.
        roi_vol: only consider voxels within a mask (e.g., a brain mask)\
            \
            the mask volume.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeWarpfieldAffineRegressionOutputs`).
    """
    params = volume_warpfield_affine_regression_params(
        warpfield=warpfield,
        affine_out=affine_out,
        flirt_out=flirt_out,
        source_volume=source_volume,
        roi_vol=roi_vol,
    )
    return volume_warpfield_affine_regression_execute(params, runner)


__all__ = [
    "VOLUME_WARPFIELD_AFFINE_REGRESSION_METADATA",
    "VolumeWarpfieldAffineRegressionFlirtOutParamsDict",
    "VolumeWarpfieldAffineRegressionFlirtOutParamsDictTagged",
    "VolumeWarpfieldAffineRegressionOutputs",
    "VolumeWarpfieldAffineRegressionParamsDict",
    "VolumeWarpfieldAffineRegressionParamsDictTagged",
    "volume_warpfield_affine_regression",
    "volume_warpfield_affine_regression_execute",
    "volume_warpfield_affine_regression_flirt_out",
    "volume_warpfield_affine_regression_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METADATA_STRING_REPLACE_METADATA = Metadata(
    id="28580214d2e8a263c11d2757b02229cfd3b15b21.workbench",
    name="metadata-string-replace",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetadataStringReplaceParamsDictNoTag = typing.TypedDict('_MetadataStringReplaceParamsDictNoTag', {
    "input-file": str,
    "find-string": str,
    "replace-string": str,
    "output-file": str,
    "case-insensitive": bool,
})
MetadataStringReplaceParamsDictTagged = typing.TypedDict('MetadataStringReplaceParamsDictTagged', {
    "@type": typing.Literal["workbench/metadata-string-replace"],
    "input-file": str,
    "find-string": str,
    "replace-string": str,
    "output-file": str,
    "case-insensitive": bool,
})
MetadataStringReplaceParamsDict = _MetadataStringReplaceParamsDictNoTag | MetadataStringReplaceParamsDictTagged


class MetadataStringReplaceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetadataStringReplaceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def metadata_string_replace_params(
    input_file: str,
    find_string: str,
    replace_string: str,
    output_file: str,
    case_insensitive: bool = False,
) -> MetadataStringReplaceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        input_file: the file to replace metadata in.
        find_string: the string to find.
        replace_string: the string to replace <find-string> with.
        output_file: output - the name to save the modified file as.
        case_insensitive: match with case variation also.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metadata-string-replace",
        "input-file": input_file,
        "find-string": find_string,
        "replace-string": replace_string,
        "output-file": output_file,
        "case-insensitive": case_insensitive,
    }
    return params


def metadata_string_replace_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetadataStringReplaceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("input-file", None) is None:
        raise StyxValidationError("`input-file` must not be None")
    if not isinstance(params["input-file"], str):
        raise StyxValidationError(f'`input-file` has the wrong type: Received `{type(params.get("input-file", None))}` expected `str`')
    if params.get("find-string", None) is None:
        raise StyxValidationError("`find-string` must not be None")
    if not isinstance(params["find-string"], str):
        raise StyxValidationError(f'`find-string` has the wrong type: Received `{type(params.get("find-string", None))}` expected `str`')
    if params.get("replace-string", None) is None:
        raise StyxValidationError("`replace-string` must not be None")
    if not isinstance(params["replace-string"], str):
        raise StyxValidationError(f'`replace-string` has the wrong type: Received `{type(params.get("replace-string", None))}` expected `str`')
    if params.get("output-file", None) is None:
        raise StyxValidationError("`output-file` must not be None")
    if not isinstance(params["output-file"], str):
        raise StyxValidationError(f'`output-file` has the wrong type: Received `{type(params.get("output-file", None))}` expected `str`')
    if params.get("case-insensitive", False) is None:
        raise StyxValidationError("`case-insensitive` must not be None")
    if not isinstance(params["case-insensitive"], bool):
        raise StyxValidationError(f'`case-insensitive` has the wrong type: Received `{type(params.get("case-insensitive", False))}` expected `bool`')


def metadata_string_replace_cargs(
    params: MetadataStringReplaceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metadata-string-replace"
    ])
    cargs.append(params.get("input-file", None))
    cargs.append(params.get("find-string", None))
    cargs.append(params.get("replace-string", None))
    cargs.append(params.get("output-file", None))
    if params.get("case-insensitive", False):
        cargs.append("-case-insensitive")
    return cargs


def metadata_string_replace_outputs(
    params: MetadataStringReplaceParamsDict,
    execution: Execution,
) -> MetadataStringReplaceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetadataStringReplaceOutputs(
        root=execution.output_file("."),
    )
    return ret


def metadata_string_replace_execute(
    params: MetadataStringReplaceParamsDict,
    runner: Runner | None = None,
) -> MetadataStringReplaceOutputs:
    """
    REPLACE A STRING IN ALL METADATA OF A FILE.
    
    Replaces all occurrences of <find-string> in the metadata and map names of
    <input-file> with <replace-string>.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetadataStringReplaceOutputs`).
    """
    metadata_string_replace_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METADATA_STRING_REPLACE_METADATA)
    params = execution.params(params)
    cargs = metadata_string_replace_cargs(params, execution)
    ret = metadata_string_replace_outputs(params, execution)
    execution.run(cargs)
    return ret


def metadata_string_replace(
    input_file: str,
    find_string: str,
    replace_string: str,
    output_file: str,
    case_insensitive: bool = False,
    runner: Runner | None = None,
) -> MetadataStringReplaceOutputs:
    """
    REPLACE A STRING IN ALL METADATA OF A FILE.
    
    Replaces all occurrences of <find-string> in the metadata and map names of
    <input-file> with <replace-string>.
    
    Args:
        input_file: the file to replace metadata in.
        find_string: the string to find.
        replace_string: the string to replace <find-string> with.
        output_file: output - the name to save the modified file as.
        case_insensitive: match with case variation also.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetadataStringReplaceOutputs`).
    """
    params = metadata_string_replace_params(
        input_file=input_file,
        find_string=find_string,
        replace_string=replace_string,
        output_file=output_file,
        case_insensitive=case_insensitive,
    )
    return metadata_string_replace_execute(params, runner)


__all__ = [
    "METADATA_STRING_REPLACE_METADATA",
    "MetadataStringReplaceOutputs",
    "MetadataStringReplaceParamsDict",
    "MetadataStringReplaceParamsDictTagged",
    "metadata_string_replace",
    "metadata_string_replace_execute",
    "metadata_string_replace_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_TFCE_METADATA = Metadata(
    id="88157dd1b5611c70b241d14e8a78bfe6302ab544.workbench",
    name="volume-tfce",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeTfcePresmoothParamsDictNoTag = typing.TypedDict('_VolumeTfcePresmoothParamsDictNoTag', {
    "kernel": float,
    "fwhm": bool,
})
VolumeTfcePresmoothParamsDictTagged = typing.TypedDict('VolumeTfcePresmoothParamsDictTagged', {
    "@type": typing.Literal["presmooth"],
    "kernel": float,
    "fwhm": bool,
})
VolumeTfcePresmoothParamsDict = _VolumeTfcePresmoothParamsDictNoTag | VolumeTfcePresmoothParamsDictTagged


_VolumeTfceParametersParamsDictNoTag = typing.TypedDict('_VolumeTfceParametersParamsDictNoTag', {
    "E": float,
    "H": float,
})
VolumeTfceParametersParamsDictTagged = typing.TypedDict('VolumeTfceParametersParamsDictTagged', {
    "@type": typing.Literal["parameters"],
    "E": float,
    "H": float,
})
VolumeTfceParametersParamsDict = _VolumeTfceParametersParamsDictNoTag | VolumeTfceParametersParamsDictTagged


_VolumeTfceParamsDictNoTag = typing.TypedDict('_VolumeTfceParamsDictNoTag', {
    "volume-in": InputPathType,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeTfcePresmoothParamsDict | None],
    "parameters": typing.NotRequired[VolumeTfceParametersParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeTfceParamsDictTagged = typing.TypedDict('VolumeTfceParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-tfce"],
    "volume-in": InputPathType,
    "volume-out": str,
    "presmooth": typing.NotRequired[VolumeTfcePresmoothParamsDict | None],
    "parameters": typing.NotRequired[VolumeTfceParametersParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "roi-volume": typing.NotRequired[InputPathType | None],
})
VolumeTfceParamsDict = _VolumeTfceParamsDictNoTag | VolumeTfceParamsDictTagged


def volume_tfce_presmooth(
    kernel: float,
    fwhm: bool = False,
) -> VolumeTfcePresmoothParamsDictTagged:
    """
    Build parameters.
    
    Args:
        kernel: the size of the gaussian smoothing kernel in mm, as sigma by\
            default.
        fwhm: smoothing kernel size is FWHM, not sigma.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "presmooth",
        "kernel": kernel,
        "fwhm": fwhm,
    }
    return params


def volume_tfce_presmooth_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeTfcePresmoothParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("kernel", None) is None:
        raise StyxValidationError("`kernel` must not be None")
    if not isinstance(params["kernel"], (float, int)):
        raise StyxValidationError(f'`kernel` has the wrong type: Received `{type(params.get("kernel", None))}` expected `float`')
    if params.get("fwhm", False) is None:
        raise StyxValidationError("`fwhm` must not be None")
    if not isinstance(params["fwhm"], bool):
        raise StyxValidationError(f'`fwhm` has the wrong type: Received `{type(params.get("fwhm", False))}` expected `bool`')


def volume_tfce_presmooth_cargs(
    params: VolumeTfcePresmoothParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-presmooth",
        str(params.get("kernel", None))
    ])
    if params.get("fwhm", False):
        cargs.append("-fwhm")
    return cargs


def volume_tfce_parameters(
    e: float,
    h: float,
) -> VolumeTfceParametersParamsDictTagged:
    """
    Build parameters.
    
    Args:
        e: exponent for cluster volume (default 0.5).
        h: exponent for threshold value (default 2.0).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "parameters",
        "E": e,
        "H": h,
    }
    return params


def volume_tfce_parameters_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeTfceParametersParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("E", None) is None:
        raise StyxValidationError("`E` must not be None")
    if not isinstance(params["E"], (float, int)):
        raise StyxValidationError(f'`E` has the wrong type: Received `{type(params.get("E", None))}` expected `float`')
    if params.get("H", None) is None:
        raise StyxValidationError("`H` must not be None")
    if not isinstance(params["H"], (float, int)):
        raise StyxValidationError(f'`H` has the wrong type: Received `{type(params.get("H", None))}` expected `float`')


def volume_tfce_parameters_cargs(
    params: VolumeTfceParametersParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-parameters",
        str(params.get("E", None)),
        str(params.get("H", None))
    ])
    return cargs


class VolumeTfceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeTfceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    volume_out: OutputPathType
    """the output volume"""


def volume_tfce_params(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeTfcePresmoothParamsDict | None = None,
    parameters: VolumeTfceParametersParamsDict | None = None,
    subvolume: str | None = None,
    roi_volume: InputPathType | None = None,
) -> VolumeTfceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the volume to run TFCE on.
        volume_out: the output volume.
        presmooth: smooth the volume before running TFCE.
        parameters: set parameters for TFCE integral.
        subvolume: select a single subvolume\
            \
            the subvolume number or name.
        roi_volume: select a region of interest to run TFCE on\
            \
            the area to run TFCE on, as a volume.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-tfce",
        "volume-in": volume_in,
        "volume-out": volume_out,
    }
    if presmooth is not None:
        params["presmooth"] = presmooth
    if parameters is not None:
        params["parameters"] = parameters
    if subvolume is not None:
        params["subvolume"] = subvolume
    if roi_volume is not None:
        params["roi-volume"] = roi_volume
    return params


def volume_tfce_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeTfceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("volume-out", None) is None:
        raise StyxValidationError("`volume-out` must not be None")
    if not isinstance(params["volume-out"], str):
        raise StyxValidationError(f'`volume-out` has the wrong type: Received `{type(params.get("volume-out", None))}` expected `str`')
    if params.get("presmooth", None) is not None:
        volume_tfce_presmooth_validate(params["presmooth"])
    if params.get("parameters", None) is not None:
        volume_tfce_parameters_validate(params["parameters"])
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], str):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str | None`')
    if params.get("roi-volume", None) is not None:
        if not isinstance(params["roi-volume"], (pathlib.Path, str)):
            raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType | None`')


def volume_tfce_cargs(
    params: VolumeTfceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-tfce"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    cargs.append(params.get("volume-out", None))
    if params.get("presmooth", None) is not None or params.get("parameters", None) is not None:
        cargs.extend([
            *(volume_tfce_presmooth_cargs(params.get("presmooth", None), execution) if (params.get("presmooth", None) is not None) else []),
            *(volume_tfce_parameters_cargs(params.get("parameters", None), execution) if (params.get("parameters", None) is not None) else [])
        ])
    if params.get("subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvolume", None)
        ])
    if params.get("roi-volume", None) is not None:
        cargs.extend([
            "-roi",
            execution.input_file(params.get("roi-volume", None))
        ])
    return cargs


def volume_tfce_outputs(
    params: VolumeTfceParamsDict,
    execution: Execution,
) -> VolumeTfceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeTfceOutputs(
        root=execution.output_file("."),
        volume_out=execution.output_file(params.get("volume-out", None)),
    )
    return ret


def volume_tfce_execute(
    params: VolumeTfceParamsDict,
    runner: Runner | None = None,
) -> VolumeTfceOutputs:
    """
    DO TFCE ON A VOLUME FILE.
    
    This command does not do any statistical analysis. Please use something like
    PALM if you are just trying to do statistics on your data.
    
    Threshold-free cluster enhancement is a method to increase the relative
    value of regions that would form clusters in a standard thresholding test.
    This is accomplished by evaluating the integral of:
    
    e(h, p)^E * h^H * dh
    
    at each voxel p, where h ranges from 0 to the maximum value in the data, and
    e(h, p) is the extent of the cluster containing voxel p at threshold h.
    Negative values are similarly enhanced by negating the data, running the
    same process, and negating the result.
    
    This method is explained in: Smith SM, Nichols TE., "Threshold-free cluster
    enhancement: addressing problems of smoothing, threshold dependence and
    localisation in cluster inference." Neuroimage. 2009 Jan 1;44(1):83-98.
    PMID: 18501637.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeTfceOutputs`).
    """
    volume_tfce_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_TFCE_METADATA)
    params = execution.params(params)
    cargs = volume_tfce_cargs(params, execution)
    ret = volume_tfce_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_tfce(
    volume_in: InputPathType,
    volume_out: str,
    presmooth: VolumeTfcePresmoothParamsDict | None = None,
    parameters: VolumeTfceParametersParamsDict | None = None,
    subvolume: str | None = None,
    roi_volume: InputPathType | None = None,
    runner: Runner | None = None,
) -> VolumeTfceOutputs:
    """
    DO TFCE ON A VOLUME FILE.
    
    This command does not do any statistical analysis. Please use something like
    PALM if you are just trying to do statistics on your data.
    
    Threshold-free cluster enhancement is a method to increase the relative
    value of regions that would form clusters in a standard thresholding test.
    This is accomplished by evaluating the integral of:
    
    e(h, p)^E * h^H * dh
    
    at each voxel p, where h ranges from 0 to the maximum value in the data, and
    e(h, p) is the extent of the cluster containing voxel p at threshold h.
    Negative values are similarly enhanced by negating the data, running the
    same process, and negating the result.
    
    This method is explained in: Smith SM, Nichols TE., "Threshold-free cluster
    enhancement: addressing problems of smoothing, threshold dependence and
    localisation in cluster inference." Neuroimage. 2009 Jan 1;44(1):83-98.
    PMID: 18501637.
    
    Args:
        volume_in: the volume to run TFCE on.
        volume_out: the output volume.
        presmooth: smooth the volume before running TFCE.
        parameters: set parameters for TFCE integral.
        subvolume: select a single subvolume\
            \
            the subvolume number or name.
        roi_volume: select a region of interest to run TFCE on\
            \
            the area to run TFCE on, as a volume.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeTfceOutputs`).
    """
    params = volume_tfce_params(
        volume_in=volume_in,
        volume_out=volume_out,
        presmooth=presmooth,
        parameters=parameters,
        subvolume=subvolume,
        roi_volume=roi_volume,
    )
    return volume_tfce_execute(params, runner)


__all__ = [
    "VOLUME_TFCE_METADATA",
    "VolumeTfceOutputs",
    "VolumeTfceParametersParamsDict",
    "VolumeTfceParametersParamsDictTagged",
    "VolumeTfceParamsDict",
    "VolumeTfceParamsDictTagged",
    "VolumeTfcePresmoothParamsDict",
    "VolumeTfcePresmoothParamsDictTagged",
    "volume_tfce",
    "volume_tfce_execute",
    "volume_tfce_parameters",
    "volume_tfce_params",
    "volume_tfce_presmooth",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

METRIC_MATH_METADATA = Metadata(
    id="c3f61d2cfe1af355592b15dccd2cddda61e44ce6.workbench",
    name="metric-math",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_MetricMathVarParamsDictNoTag = typing.TypedDict('_MetricMathVarParamsDictNoTag', {
    "name": str,
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
    "repeat": bool,
})
MetricMathVarParamsDictTagged = typing.TypedDict('MetricMathVarParamsDictTagged', {
    "@type": typing.Literal["var"],
    "name": str,
    "metric": InputPathType,
    "column": typing.NotRequired[str | None],
    "repeat": bool,
})
MetricMathVarParamsDict = _MetricMathVarParamsDictNoTag | MetricMathVarParamsDictTagged


_MetricMathParamsDictNoTag = typing.TypedDict('_MetricMathParamsDictNoTag', {
    "expression": str,
    "metric-out": str,
    "var": typing.NotRequired[list[MetricMathVarParamsDict] | None],
    "replace": typing.NotRequired[float | None],
})
MetricMathParamsDictTagged = typing.TypedDict('MetricMathParamsDictTagged', {
    "@type": typing.Literal["workbench/metric-math"],
    "expression": str,
    "metric-out": str,
    "var": typing.NotRequired[list[MetricMathVarParamsDict] | None],
    "replace": typing.NotRequired[float | None],
})
MetricMathParamsDict = _MetricMathParamsDictNoTag | MetricMathParamsDictTagged


def metric_math_var(
    name: str,
    metric: InputPathType,
    column: str | None = None,
    repeat: bool = False,
) -> MetricMathVarParamsDictTagged:
    """
    Build parameters.
    
    Args:
        name: the name of the variable, as used in the expression.
        metric: the metric file to use as this variable.
        column: select a single column\
            \
            the column number or name.
        repeat: reuse a single column for each column of calculation.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "var",
        "name": name,
        "metric": metric,
        "repeat": repeat,
    }
    if column is not None:
        params["column"] = column
    return params


def metric_math_var_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMathVarParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("name", None) is None:
        raise StyxValidationError("`name` must not be None")
    if not isinstance(params["name"], str):
        raise StyxValidationError(f'`name` has the wrong type: Received `{type(params.get("name", None))}` expected `str`')
    if params.get("metric", None) is None:
        raise StyxValidationError("`metric` must not be None")
    if not isinstance(params["metric"], (pathlib.Path, str)):
        raise StyxValidationError(f'`metric` has the wrong type: Received `{type(params.get("metric", None))}` expected `InputPathType`')
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("repeat", False) is None:
        raise StyxValidationError("`repeat` must not be None")
    if not isinstance(params["repeat"], bool):
        raise StyxValidationError(f'`repeat` has the wrong type: Received `{type(params.get("repeat", False))}` expected `bool`')


def metric_math_var_cargs(
    params: MetricMathVarParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-var",
        params.get("name", None),
        execution.input_file(params.get("metric", None))
    ])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("repeat", False):
        cargs.append("-repeat")
    return cargs


class MetricMathOutputs(typing.NamedTuple):
    """
    Output object returned when calling `MetricMathParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    metric_out: OutputPathType
    """the output metric"""


def metric_math_params(
    expression: str,
    metric_out: str,
    var: list[MetricMathVarParamsDict] | None = None,
    replace: float | None = None,
) -> MetricMathParamsDictTagged:
    """
    Build parameters.
    
    Args:
        expression: the expression to evaluate, in quotes.
        metric_out: the output metric.
        var: a metric to use as a variable.
        replace: replace NaN results with a value\
            \
            value to replace NaN with.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/metric-math",
        "expression": expression,
        "metric-out": metric_out,
    }
    if var is not None:
        params["var"] = var
    if replace is not None:
        params["replace"] = replace
    return params


def metric_math_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `MetricMathParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("expression", None) is None:
        raise StyxValidationError("`expression` must not be None")
    if not isinstance(params["expression"], str):
        raise StyxValidationError(f'`expression` has the wrong type: Received `{type(params.get("expression", None))}` expected `str`')
    if params.get("metric-out", None) is None:
        raise StyxValidationError("`metric-out` must not be None")
    if not isinstance(params["metric-out"], str):
        raise StyxValidationError(f'`metric-out` has the wrong type: Received `{type(params.get("metric-out", None))}` expected `str`')
    if params.get("var", None) is not None:
        if not isinstance(params["var"], list):
            raise StyxValidationError(f'`var` has the wrong type: Received `{type(params.get("var", None))}` expected `list[MetricMathVarParamsDict] | None`')
        for e in params["var"]:
            metric_math_var_validate(e)
    if params.get("replace", None) is not None:
        if not isinstance(params["replace"], (float, int)):
            raise StyxValidationError(f'`replace` has the wrong type: Received `{type(params.get("replace", None))}` expected `float | None`')


def metric_math_cargs(
    params: MetricMathParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-metric-math"
    ])
    cargs.append(params.get("expression", None))
    cargs.append(params.get("metric-out", None))
    if params.get("var", None) is not None:
        cargs.extend([a for c in [metric_math_var_cargs(s, execution) for s in params.get("var", None)] for a in c])
    if params.get("replace", None) is not None:
        cargs.extend([
            "-fixnan",
            str(params.get("replace", None))
        ])
    return cargs


def metric_math_outputs(
    params: MetricMathParamsDict,
    execution: Execution,
) -> MetricMathOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = MetricMathOutputs(
        root=execution.output_file("."),
        metric_out=execution.output_file(params.get("metric-out", None)),
    )
    return ret


def metric_math_execute(
    params: MetricMathParamsDict,
    runner: Runner | None = None,
) -> MetricMathOutputs:
    """
    EVALUATE EXPRESSION ON METRIC FILES.
    
    This command evaluates <expression> at each surface vertex independently.
    There must be at least one -var option (to get the structure, number of
    vertices, and number of columns from), even if the <name> specified in it
    isn't used in <expression>. All metrics must have the same number of
    vertices. Filenames are not valid in <expression>, use a variable name and a
    -var option with matching <name> to specify an input file. If the -column
    option is given to any -var option, only one column is used from that file.
    If -repeat is specified, the file must either have only one column, or have
    the -column option specified. All files that don't use -repeat must have the
    same number of columns requested to be used. The format of <expression> is
    as follows:
    
    Expressions consist of constants, variables, operators, parentheses, and
    functions, in infix notation, such as 'exp(-x + 3) * scale'. Variables are
    strings of any length, using the characters a-z, A-Z, 0-9, and _, but may
    not take the name of a named constant. Currently, there is only one named
    constant, PI. The operators are +, -, *, /, ^, >, <, >=, <=, ==, !=, !, &&,
    ||. These behave as in C, except that ^ is exponentiation, i.e. pow(x, y),
    and takes higher precedence than other binary operators (also, '-3^-4^-5'
    means '-(3^(-(4^-5)))'). The <=, >=, ==, and != operators are given a small
    amount of wiggle room, equal to one millionth of the smaller of the absolute
    values of the values being compared.
    
    Comparison and logical operators return 0 or 1, you can do masking with
    expressions like 'x * (mask > 0)'. For all logical operators, an input is
    considered true iff it is greater than 0. The expression '0 < x < 5' is not
    syntactically wrong, but it will NOT do what is desired, because it is
    evaluated left to right, i.e. '((0 < x) < 5)', which will always return 1,
    as both possible results of a comparison are less than 5. A warning is
    generated if an expression of this type is detected. Use something like 'x >
    0 && x < 5' to get the desired behavior.
    
    Whitespace between elements is ignored, ' sin ( 2 * x ) ' is equivalent to
    'sin(2*x)', but 's in(2*x)' is an error. Implied multiplication is not
    allowed, the expression '2x' will be parsed as a variable. Parentheses are
    (), do not use [] or {}. Functions require parentheses, the expression 'sin
    x' is an error.
    
    The following functions are supported:
    
    sin: 1 argument, the sine of the argument (units are radians)
    cos: 1 argument, the cosine of the argument (units are radians)
    tan: 1 argument, the tangent of the argument (units are radians)
    asin: 1 argument, the inverse of sine of the argument, in radians
    acos: 1 argument, the inverse of cosine of the argument, in radians
    atan: 1 argument, the inverse of tangent of the argument, in radians
    atan2: 2 arguments, atan2(y, x) returns the inverse of tangent of (y/x), in
    radians, determining quadrant by the sign of both arguments
    sinh: 1 argument, the hyperbolic sine of the argument
    cosh: 1 argument, the hyperbolic cosine of the argument
    tanh: 1 argument, the hyperbolic tangent of the argument
    asinh: 1 argument, the inverse hyperbolic sine of the argument
    acosh: 1 argument, the inverse hyperbolic cosine of the argument
    atanh: 1 argument, the inverse hyperbolic tangent of the argument
    sinc: 1 argument, sinc(0) = 1, sin(x) / x otherwise
    ln: 1 argument, the natural logarithm of the argument
    exp: 1 argument, the constant e raised to the power of the argument
    log: 1 argument, the base 10 logarithm of the argument
    log2: 1 argument, the base 2 logarithm of the argument
    sqrt: 1 argument, the square root of the argument
    abs: 1 argument, the absolute value of the argument
    floor: 1 argument, the largest integer not greater than the argument
    round: 1 argument, the nearest integer, with ties rounded away from zero
    ceil: 1 argument, the smallest integer not less than the argument
    min: 2 arguments, min(x, y) returns y if (x > y), x otherwise
    max: 2 arguments, max(x, y) returns y if (x < y), x otherwise
    mod: 2 arguments, mod(x, y) = x - y * floor(x / y), or 0 if y == 0
    clamp: 3 arguments, clamp(x, low, high) = min(max(x, low), high)
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMathOutputs`).
    """
    metric_math_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(METRIC_MATH_METADATA)
    params = execution.params(params)
    cargs = metric_math_cargs(params, execution)
    ret = metric_math_outputs(params, execution)
    execution.run(cargs)
    return ret


def metric_math(
    expression: str,
    metric_out: str,
    var: list[MetricMathVarParamsDict] | None = None,
    replace: float | None = None,
    runner: Runner | None = None,
) -> MetricMathOutputs:
    """
    EVALUATE EXPRESSION ON METRIC FILES.
    
    This command evaluates <expression> at each surface vertex independently.
    There must be at least one -var option (to get the structure, number of
    vertices, and number of columns from), even if the <name> specified in it
    isn't used in <expression>. All metrics must have the same number of
    vertices. Filenames are not valid in <expression>, use a variable name and a
    -var option with matching <name> to specify an input file. If the -column
    option is given to any -var option, only one column is used from that file.
    If -repeat is specified, the file must either have only one column, or have
    the -column option specified. All files that don't use -repeat must have the
    same number of columns requested to be used. The format of <expression> is
    as follows:
    
    Expressions consist of constants, variables, operators, parentheses, and
    functions, in infix notation, such as 'exp(-x + 3) * scale'. Variables are
    strings of any length, using the characters a-z, A-Z, 0-9, and _, but may
    not take the name of a named constant. Currently, there is only one named
    constant, PI. The operators are +, -, *, /, ^, >, <, >=, <=, ==, !=, !, &&,
    ||. These behave as in C, except that ^ is exponentiation, i.e. pow(x, y),
    and takes higher precedence than other binary operators (also, '-3^-4^-5'
    means '-(3^(-(4^-5)))'). The <=, >=, ==, and != operators are given a small
    amount of wiggle room, equal to one millionth of the smaller of the absolute
    values of the values being compared.
    
    Comparison and logical operators return 0 or 1, you can do masking with
    expressions like 'x * (mask > 0)'. For all logical operators, an input is
    considered true iff it is greater than 0. The expression '0 < x < 5' is not
    syntactically wrong, but it will NOT do what is desired, because it is
    evaluated left to right, i.e. '((0 < x) < 5)', which will always return 1,
    as both possible results of a comparison are less than 5. A warning is
    generated if an expression of this type is detected. Use something like 'x >
    0 && x < 5' to get the desired behavior.
    
    Whitespace between elements is ignored, ' sin ( 2 * x ) ' is equivalent to
    'sin(2*x)', but 's in(2*x)' is an error. Implied multiplication is not
    allowed, the expression '2x' will be parsed as a variable. Parentheses are
    (), do not use [] or {}. Functions require parentheses, the expression 'sin
    x' is an error.
    
    The following functions are supported:
    
    sin: 1 argument, the sine of the argument (units are radians)
    cos: 1 argument, the cosine of the argument (units are radians)
    tan: 1 argument, the tangent of the argument (units are radians)
    asin: 1 argument, the inverse of sine of the argument, in radians
    acos: 1 argument, the inverse of cosine of the argument, in radians
    atan: 1 argument, the inverse of tangent of the argument, in radians
    atan2: 2 arguments, atan2(y, x) returns the inverse of tangent of (y/x), in
    radians, determining quadrant by the sign of both arguments
    sinh: 1 argument, the hyperbolic sine of the argument
    cosh: 1 argument, the hyperbolic cosine of the argument
    tanh: 1 argument, the hyperbolic tangent of the argument
    asinh: 1 argument, the inverse hyperbolic sine of the argument
    acosh: 1 argument, the inverse hyperbolic cosine of the argument
    atanh: 1 argument, the inverse hyperbolic tangent of the argument
    sinc: 1 argument, sinc(0) = 1, sin(x) / x otherwise
    ln: 1 argument, the natural logarithm of the argument
    exp: 1 argument, the constant e raised to the power of the argument
    log: 1 argument, the base 10 logarithm of the argument
    log2: 1 argument, the base 2 logarithm of the argument
    sqrt: 1 argument, the square root of the argument
    abs: 1 argument, the absolute value of the argument
    floor: 1 argument, the largest integer not greater than the argument
    round: 1 argument, the nearest integer, with ties rounded away from zero
    ceil: 1 argument, the smallest integer not less than the argument
    min: 2 arguments, min(x, y) returns y if (x > y), x otherwise
    max: 2 arguments, max(x, y) returns y if (x < y), x otherwise
    mod: 2 arguments, mod(x, y) = x - y * floor(x / y), or 0 if y == 0
    clamp: 3 arguments, clamp(x, low, high) = min(max(x, low), high)
    .
    
    Args:
        expression: the expression to evaluate, in quotes.
        metric_out: the output metric.
        var: a metric to use as a variable.
        replace: replace NaN results with a value\
            \
            value to replace NaN with.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `MetricMathOutputs`).
    """
    params = metric_math_params(
        expression=expression,
        metric_out=metric_out,
        var=var,
        replace=replace,
    )
    return metric_math_execute(params, runner)


__all__ = [
    "METRIC_MATH_METADATA",
    "MetricMathOutputs",
    "MetricMathParamsDict",
    "MetricMathParamsDictTagged",
    "MetricMathVarParamsDict",
    "MetricMathVarParamsDictTagged",
    "metric_math",
    "metric_math_execute",
    "metric_math_params",
    "metric_math_var",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

VOLUME_STATS_METADATA = Metadata(
    id="ec1af7e3dc9990ebe00878d35008f857657ae3d6.workbench",
    name="volume-stats",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_VolumeStatsRoiParamsDictNoTag = typing.TypedDict('_VolumeStatsRoiParamsDictNoTag', {
    "roi-volume": InputPathType,
    "match-maps": bool,
})
VolumeStatsRoiParamsDictTagged = typing.TypedDict('VolumeStatsRoiParamsDictTagged', {
    "@type": typing.Literal["roi"],
    "roi-volume": InputPathType,
    "match-maps": bool,
})
VolumeStatsRoiParamsDict = _VolumeStatsRoiParamsDictNoTag | VolumeStatsRoiParamsDictTagged


_VolumeStatsParamsDictNoTag = typing.TypedDict('_VolumeStatsParamsDictNoTag', {
    "volume-in": InputPathType,
    "roi": typing.NotRequired[VolumeStatsRoiParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "percent": typing.NotRequired[float | None],
    "operation": typing.NotRequired[str | None],
    "show-map-name": bool,
})
VolumeStatsParamsDictTagged = typing.TypedDict('VolumeStatsParamsDictTagged', {
    "@type": typing.Literal["workbench/volume-stats"],
    "volume-in": InputPathType,
    "roi": typing.NotRequired[VolumeStatsRoiParamsDict | None],
    "subvolume": typing.NotRequired[str | None],
    "percent": typing.NotRequired[float | None],
    "operation": typing.NotRequired[str | None],
    "show-map-name": bool,
})
VolumeStatsParamsDict = _VolumeStatsParamsDictNoTag | VolumeStatsParamsDictTagged


def volume_stats_roi(
    roi_volume: InputPathType,
    match_maps: bool = False,
) -> VolumeStatsRoiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        roi_volume: the roi, as a volume file.
        match_maps: each subvolume of input uses the corresponding subvolume\
            from the roi file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "roi",
        "roi-volume": roi_volume,
        "match-maps": match_maps,
    }
    return params


def volume_stats_roi_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeStatsRoiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("roi-volume", None) is None:
        raise StyxValidationError("`roi-volume` must not be None")
    if not isinstance(params["roi-volume"], (pathlib.Path, str)):
        raise StyxValidationError(f'`roi-volume` has the wrong type: Received `{type(params.get("roi-volume", None))}` expected `InputPathType`')
    if params.get("match-maps", False) is None:
        raise StyxValidationError("`match-maps` must not be None")
    if not isinstance(params["match-maps"], bool):
        raise StyxValidationError(f'`match-maps` has the wrong type: Received `{type(params.get("match-maps", False))}` expected `bool`')


def volume_stats_roi_cargs(
    params: VolumeStatsRoiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-roi",
        execution.input_file(params.get("roi-volume", None))
    ])
    if params.get("match-maps", False):
        cargs.append("-match-maps")
    return cargs


class VolumeStatsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `VolumeStatsParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def volume_stats_params(
    volume_in: InputPathType,
    roi: VolumeStatsRoiParamsDict | None = None,
    subvolume: str | None = None,
    percent: float | None = None,
    operation: str | None = None,
    show_map_name: bool = False,
) -> VolumeStatsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        volume_in: the input volume.
        roi: only consider data inside an roi.
        subvolume: only display output for one subvolume\
            \
            the subvolume number or name.
        percent: give the value at a percentile\
            \
            the percentile to find, must be between 0 and 100.
        operation: use a reduction operation\
            \
            the reduction operation.
        show_map_name: print map index and name before each output.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/volume-stats",
        "volume-in": volume_in,
        "show-map-name": show_map_name,
    }
    if roi is not None:
        params["roi"] = roi
    if subvolume is not None:
        params["subvolume"] = subvolume
    if percent is not None:
        params["percent"] = percent
    if operation is not None:
        params["operation"] = operation
    return params


def volume_stats_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `VolumeStatsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("volume-in", None) is None:
        raise StyxValidationError("`volume-in` must not be None")
    if not isinstance(params["volume-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`volume-in` has the wrong type: Received `{type(params.get("volume-in", None))}` expected `InputPathType`')
    if params.get("roi", None) is not None:
        volume_stats_roi_validate(params["roi"])
    if params.get("subvolume", None) is not None:
        if not isinstance(params["subvolume"], str):
            raise StyxValidationError(f'`subvolume` has the wrong type: Received `{type(params.get("subvolume", None))}` expected `str | None`')
    if params.get("percent", None) is not None:
        if not isinstance(params["percent"], (float, int)):
            raise StyxValidationError(f'`percent` has the wrong type: Received `{type(params.get("percent", None))}` expected `float | None`')
    if params.get("operation", None) is not None:
        if not isinstance(params["operation"], str):
            raise StyxValidationError(f'`operation` has the wrong type: Received `{type(params.get("operation", None))}` expected `str | None`')
    if params.get("show-map-name", False) is None:
        raise StyxValidationError("`show-map-name` must not be None")
    if not isinstance(params["show-map-name"], bool):
        raise StyxValidationError(f'`show-map-name` has the wrong type: Received `{type(params.get("show-map-name", False))}` expected `bool`')


def volume_stats_cargs(
    params: VolumeStatsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-volume-stats"
    ])
    cargs.append(execution.input_file(params.get("volume-in", None)))
    if params.get("roi", None) is not None:
        cargs.extend(volume_stats_roi_cargs(params.get("roi", None), execution))
    if params.get("subvolume", None) is not None:
        cargs.extend([
            "-subvolume",
            params.get("subvolume", None)
        ])
    if params.get("percent", None) is not None:
        cargs.extend([
            "-percentile",
            str(params.get("percent", None))
        ])
    if params.get("operation", None) is not None:
        cargs.extend([
            "-reduce",
            params.get("operation", None)
        ])
    if params.get("show-map-name", False):
        cargs.append("-show-map-name")
    return cargs


def volume_stats_outputs(
    params: VolumeStatsParamsDict,
    execution: Execution,
) -> VolumeStatsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = VolumeStatsOutputs(
        root=execution.output_file("."),
    )
    return ret


def volume_stats_execute(
    params: VolumeStatsParamsDict,
    runner: Runner | None = None,
) -> VolumeStatsOutputs:
    """
    SPATIAL STATISTICS ON A VOLUME FILE.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified reduction or percentile operation. Use -subvolume to only give
    output for a single subvolume. If the -roi option is used without
    -match-maps, then each line will contain as many numbers as there are maps
    in the ROI file, separated by tab characters. Exactly one of -reduce or
    -percentile must be specified.
    
    The argument to the -reduce option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    .
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeStatsOutputs`).
    """
    volume_stats_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(VOLUME_STATS_METADATA)
    params = execution.params(params)
    cargs = volume_stats_cargs(params, execution)
    ret = volume_stats_outputs(params, execution)
    execution.run(cargs)
    return ret


def volume_stats(
    volume_in: InputPathType,
    roi: VolumeStatsRoiParamsDict | None = None,
    subvolume: str | None = None,
    percent: float | None = None,
    operation: str | None = None,
    show_map_name: bool = False,
    runner: Runner | None = None,
) -> VolumeStatsOutputs:
    """
    SPATIAL STATISTICS ON A VOLUME FILE.
    
    For each subvolume of the input, a line of text is printed, resulting from
    the specified reduction or percentile operation. Use -subvolume to only give
    output for a single subvolume. If the -roi option is used without
    -match-maps, then each line will contain as many numbers as there are maps
    in the ROI file, separated by tab characters. Exactly one of -reduce or
    -percentile must be specified.
    
    The argument to the -reduce option must be one of the following:
    
    MAX: the maximum value
    MIN: the minimum value
    INDEXMAX: the 1-based index of the maximum value
    INDEXMIN: the 1-based index of the minimum value
    SUM: add all values
    PRODUCT: multiply all values
    MEAN: the mean of the data
    STDEV: the standard deviation (N denominator)
    SAMPSTDEV: the sample standard deviation (N-1 denominator)
    VARIANCE: the variance of the data
    TSNR: mean divided by sample standard deviation (N-1 denominator)
    COV: sample standard deviation (N-1 denominator) divided by mean
    L2NORM: square root of sum of squares
    MEDIAN: the median of the data
    MODE: the mode of the data
    COUNT_NONZERO: the number of nonzero elements in the data
    .
    
    Args:
        volume_in: the input volume.
        roi: only consider data inside an roi.
        subvolume: only display output for one subvolume\
            \
            the subvolume number or name.
        percent: give the value at a percentile\
            \
            the percentile to find, must be between 0 and 100.
        operation: use a reduction operation\
            \
            the reduction operation.
        show_map_name: print map index and name before each output.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `VolumeStatsOutputs`).
    """
    params = volume_stats_params(
        volume_in=volume_in,
        roi=roi,
        subvolume=subvolume,
        percent=percent,
        operation=operation,
        show_map_name=show_map_name,
    )
    return volume_stats_execute(params, runner)


__all__ = [
    "VOLUME_STATS_METADATA",
    "VolumeStatsOutputs",
    "VolumeStatsParamsDict",
    "VolumeStatsParamsDictTagged",
    "VolumeStatsRoiParamsDict",
    "VolumeStatsRoiParamsDictTagged",
    "volume_stats",
    "volume_stats_execute",
    "volume_stats_params",
    "volume_stats_roi",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_TO_SURFACE_3D_DISTANCE_METADATA = Metadata(
    id="8c740646d34e5a61db7e9f5f4bf71bcd927fb141.workbench",
    name="surface-to-surface-3d-distance",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceToSurface3dDistanceVectorsParamsDictNoTag = typing.TypedDict('_SurfaceToSurface3dDistanceVectorsParamsDictNoTag', {
    "vectors-out": str,
})
SurfaceToSurface3dDistanceVectorsParamsDictTagged = typing.TypedDict('SurfaceToSurface3dDistanceVectorsParamsDictTagged', {
    "@type": typing.Literal["vectors"],
    "vectors-out": str,
})
SurfaceToSurface3dDistanceVectorsParamsDict = _SurfaceToSurface3dDistanceVectorsParamsDictNoTag | SurfaceToSurface3dDistanceVectorsParamsDictTagged


_SurfaceToSurface3dDistanceParamsDictNoTag = typing.TypedDict('_SurfaceToSurface3dDistanceParamsDictNoTag', {
    "surface-comp": InputPathType,
    "surface-ref": InputPathType,
    "dists-out": str,
    "vectors": typing.NotRequired[SurfaceToSurface3dDistanceVectorsParamsDict | None],
})
SurfaceToSurface3dDistanceParamsDictTagged = typing.TypedDict('SurfaceToSurface3dDistanceParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-to-surface-3d-distance"],
    "surface-comp": InputPathType,
    "surface-ref": InputPathType,
    "dists-out": str,
    "vectors": typing.NotRequired[SurfaceToSurface3dDistanceVectorsParamsDict | None],
})
SurfaceToSurface3dDistanceParamsDict = _SurfaceToSurface3dDistanceParamsDictNoTag | SurfaceToSurface3dDistanceParamsDictTagged


class SurfaceToSurface3dDistanceVectorsOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceToSurface3dDistanceVectorsParamsDict | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    vectors_out: OutputPathType
    """the output vectors"""


def surface_to_surface_3d_distance_vectors(
    vectors_out: str,
) -> SurfaceToSurface3dDistanceVectorsParamsDictTagged:
    """
    Build parameters.
    
    Args:
        vectors_out: the output vectors.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "vectors",
        "vectors-out": vectors_out,
    }
    return params


def surface_to_surface_3d_distance_vectors_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceToSurface3dDistanceVectorsParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("vectors-out", None) is None:
        raise StyxValidationError("`vectors-out` must not be None")
    if not isinstance(params["vectors-out"], str):
        raise StyxValidationError(f'`vectors-out` has the wrong type: Received `{type(params.get("vectors-out", None))}` expected `str`')


def surface_to_surface_3d_distance_vectors_cargs(
    params: SurfaceToSurface3dDistanceVectorsParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-vectors",
        params.get("vectors-out", None)
    ])
    return cargs


def surface_to_surface_3d_distance_vectors_outputs(
    params: SurfaceToSurface3dDistanceVectorsParamsDict,
    execution: Execution,
) -> SurfaceToSurface3dDistanceVectorsOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceToSurface3dDistanceVectorsOutputs(
        root=execution.output_file("."),
        vectors_out=execution.output_file(params.get("vectors-out", None)),
    )
    return ret


class SurfaceToSurface3dDistanceOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceToSurface3dDistanceParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    dists_out: OutputPathType
    """the output distances"""
    vectors: SurfaceToSurface3dDistanceVectorsOutputs | None
    """Outputs from `surface_to_surface_3d_distance_vectors_outputs`."""


def surface_to_surface_3d_distance_params(
    surface_comp: InputPathType,
    surface_ref: InputPathType,
    dists_out: str,
    vectors: SurfaceToSurface3dDistanceVectorsParamsDict | None = None,
) -> SurfaceToSurface3dDistanceParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_comp: the surface to compare to the reference.
        surface_ref: the surface to use as the reference.
        dists_out: the output distances.
        vectors: output the displacement vectors.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-to-surface-3d-distance",
        "surface-comp": surface_comp,
        "surface-ref": surface_ref,
        "dists-out": dists_out,
    }
    if vectors is not None:
        params["vectors"] = vectors
    return params


def surface_to_surface_3d_distance_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceToSurface3dDistanceParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface-comp", None) is None:
        raise StyxValidationError("`surface-comp` must not be None")
    if not isinstance(params["surface-comp"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-comp` has the wrong type: Received `{type(params.get("surface-comp", None))}` expected `InputPathType`')
    if params.get("surface-ref", None) is None:
        raise StyxValidationError("`surface-ref` must not be None")
    if not isinstance(params["surface-ref"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface-ref` has the wrong type: Received `{type(params.get("surface-ref", None))}` expected `InputPathType`')
    if params.get("dists-out", None) is None:
        raise StyxValidationError("`dists-out` must not be None")
    if not isinstance(params["dists-out"], str):
        raise StyxValidationError(f'`dists-out` has the wrong type: Received `{type(params.get("dists-out", None))}` expected `str`')
    if params.get("vectors", None) is not None:
        surface_to_surface_3d_distance_vectors_validate(params["vectors"])


def surface_to_surface_3d_distance_cargs(
    params: SurfaceToSurface3dDistanceParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-to-surface-3d-distance"
    ])
    cargs.append(execution.input_file(params.get("surface-comp", None)))
    cargs.append(execution.input_file(params.get("surface-ref", None)))
    cargs.append(params.get("dists-out", None))
    if params.get("vectors", None) is not None:
        cargs.extend(surface_to_surface_3d_distance_vectors_cargs(params.get("vectors", None), execution))
    return cargs


def surface_to_surface_3d_distance_outputs(
    params: SurfaceToSurface3dDistanceParamsDict,
    execution: Execution,
) -> SurfaceToSurface3dDistanceOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceToSurface3dDistanceOutputs(
        root=execution.output_file("."),
        dists_out=execution.output_file(params.get("dists-out", None)),
        vectors=surface_to_surface_3d_distance_vectors_outputs(params.get("vectors"), execution) if params.get("vectors") else None,
    )
    return ret


def surface_to_surface_3d_distance_execute(
    params: SurfaceToSurface3dDistanceParamsDict,
    runner: Runner | None = None,
) -> SurfaceToSurface3dDistanceOutputs:
    """
    COMPUTE DISTANCE BETWEEN CORRESPONDING VERTICES.
    
    Computes the vector difference between the vertices of each surface with the
    same index, as (comp - ref), and output the magnitudes, and optionally the
    displacement vectors.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceToSurface3dDistanceOutputs`).
    """
    surface_to_surface_3d_distance_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_TO_SURFACE_3D_DISTANCE_METADATA)
    params = execution.params(params)
    cargs = surface_to_surface_3d_distance_cargs(params, execution)
    ret = surface_to_surface_3d_distance_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_to_surface_3d_distance(
    surface_comp: InputPathType,
    surface_ref: InputPathType,
    dists_out: str,
    vectors: SurfaceToSurface3dDistanceVectorsParamsDict | None = None,
    runner: Runner | None = None,
) -> SurfaceToSurface3dDistanceOutputs:
    """
    COMPUTE DISTANCE BETWEEN CORRESPONDING VERTICES.
    
    Computes the vector difference between the vertices of each surface with the
    same index, as (comp - ref), and output the magnitudes, and optionally the
    displacement vectors.
    
    Args:
        surface_comp: the surface to compare to the reference.
        surface_ref: the surface to use as the reference.
        dists_out: the output distances.
        vectors: output the displacement vectors.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceToSurface3dDistanceOutputs`).
    """
    params = surface_to_surface_3d_distance_params(
        surface_comp=surface_comp,
        surface_ref=surface_ref,
        dists_out=dists_out,
        vectors=vectors,
    )
    return surface_to_surface_3d_distance_execute(params, runner)


__all__ = [
    "SURFACE_TO_SURFACE_3D_DISTANCE_METADATA",
    "SurfaceToSurface3dDistanceOutputs",
    "SurfaceToSurface3dDistanceParamsDict",
    "SurfaceToSurface3dDistanceParamsDictTagged",
    "SurfaceToSurface3dDistanceVectorsOutputs",
    "SurfaceToSurface3dDistanceVectorsParamsDict",
    "SurfaceToSurface3dDistanceVectorsParamsDictTagged",
    "surface_to_surface_3d_distance",
    "surface_to_surface_3d_distance_execute",
    "surface_to_surface_3d_distance_params",
    "surface_to_surface_3d_distance_vectors",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_LABEL_TO_BORDER_METADATA = Metadata(
    id="ad9549859ecd3b88b8cca8704b00cceb77188e6a.workbench",
    name="cifti-label-to-border",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiLabelToBorderBorderParamsDictNoTag = typing.TypedDict('_CiftiLabelToBorderBorderParamsDictNoTag', {
    "surface": InputPathType,
    "border-out": str,
})
CiftiLabelToBorderBorderParamsDictTagged = typing.TypedDict('CiftiLabelToBorderBorderParamsDictTagged', {
    "@type": typing.Literal["border"],
    "surface": InputPathType,
    "border-out": str,
})
CiftiLabelToBorderBorderParamsDict = _CiftiLabelToBorderBorderParamsDictNoTag | CiftiLabelToBorderBorderParamsDictTagged


_CiftiLabelToBorderParamsDictNoTag = typing.TypedDict('_CiftiLabelToBorderParamsDictNoTag', {
    "cifti-in": InputPathType,
    "border": typing.NotRequired[list[CiftiLabelToBorderBorderParamsDict] | None],
    "column": typing.NotRequired[str | None],
    "fraction": typing.NotRequired[float | None],
})
CiftiLabelToBorderParamsDictTagged = typing.TypedDict('CiftiLabelToBorderParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-label-to-border"],
    "cifti-in": InputPathType,
    "border": typing.NotRequired[list[CiftiLabelToBorderBorderParamsDict] | None],
    "column": typing.NotRequired[str | None],
    "fraction": typing.NotRequired[float | None],
})
CiftiLabelToBorderParamsDict = _CiftiLabelToBorderParamsDictNoTag | CiftiLabelToBorderParamsDictTagged


class CiftiLabelToBorderBorderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `list[CiftiLabelToBorderBorderParamsDict] | None(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    border_out: OutputPathType
    """the output border file"""


def cifti_label_to_border_border(
    surface: InputPathType,
    border_out: str,
) -> CiftiLabelToBorderBorderParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface: the surface to use for neighbor and structure information.
        border_out: the output border file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "border",
        "surface": surface,
        "border-out": border_out,
    }
    return params


def cifti_label_to_border_border_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelToBorderBorderParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("surface", None) is None:
        raise StyxValidationError("`surface` must not be None")
    if not isinstance(params["surface"], (pathlib.Path, str)):
        raise StyxValidationError(f'`surface` has the wrong type: Received `{type(params.get("surface", None))}` expected `InputPathType`')
    if params.get("border-out", None) is None:
        raise StyxValidationError("`border-out` must not be None")
    if not isinstance(params["border-out"], str):
        raise StyxValidationError(f'`border-out` has the wrong type: Received `{type(params.get("border-out", None))}` expected `str`')


def cifti_label_to_border_border_cargs(
    params: CiftiLabelToBorderBorderParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-border",
        execution.input_file(params.get("surface", None)),
        params.get("border-out", None)
    ])
    return cargs


def cifti_label_to_border_border_outputs(
    params: CiftiLabelToBorderBorderParamsDict,
    execution: Execution,
) -> CiftiLabelToBorderBorderOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelToBorderBorderOutputs(
        root=execution.output_file("."),
        border_out=execution.output_file(params.get("border-out", None)),
    )
    return ret


class CiftiLabelToBorderOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiLabelToBorderParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    border: list[CiftiLabelToBorderBorderOutputs] | None
    """Outputs from `cifti_label_to_border_border_outputs`.This is a list of
    outputs with the same length and order as the inputs."""


def cifti_label_to_border_params(
    cifti_in: InputPathType,
    border: list[CiftiLabelToBorderBorderParamsDict] | None = None,
    column: str | None = None,
    fraction: float | None = None,
) -> CiftiLabelToBorderParamsDictTagged:
    """
    Build parameters.
    
    Args:
        cifti_in: the input cifti dlabel file.
        border: specify output file for a surface structure.
        column: select a single column\
            \
            the column number or name.
        fraction: set how far along the edge border points are drawn\
            \
            fraction along edge from inside vertex (default 0.33).
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-label-to-border",
        "cifti-in": cifti_in,
    }
    if border is not None:
        params["border"] = border
    if column is not None:
        params["column"] = column
    if fraction is not None:
        params["fraction"] = fraction
    return params


def cifti_label_to_border_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelToBorderParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("cifti-in", None) is None:
        raise StyxValidationError("`cifti-in` must not be None")
    if not isinstance(params["cifti-in"], (pathlib.Path, str)):
        raise StyxValidationError(f'`cifti-in` has the wrong type: Received `{type(params.get("cifti-in", None))}` expected `InputPathType`')
    if params.get("border", None) is not None:
        if not isinstance(params["border"], list):
            raise StyxValidationError(f'`border` has the wrong type: Received `{type(params.get("border", None))}` expected `list[CiftiLabelToBorderBorderParamsDict] | None`')
        for e in params["border"]:
            cifti_label_to_border_border_validate(e)
    if params.get("column", None) is not None:
        if not isinstance(params["column"], str):
            raise StyxValidationError(f'`column` has the wrong type: Received `{type(params.get("column", None))}` expected `str | None`')
    if params.get("fraction", None) is not None:
        if not isinstance(params["fraction"], (float, int)):
            raise StyxValidationError(f'`fraction` has the wrong type: Received `{type(params.get("fraction", None))}` expected `float | None`')


def cifti_label_to_border_cargs(
    params: CiftiLabelToBorderParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-label-to-border"
    ])
    cargs.append(execution.input_file(params.get("cifti-in", None)))
    if params.get("border", None) is not None:
        cargs.extend([a for c in [cifti_label_to_border_border_cargs(s, execution) for s in params.get("border", None)] for a in c])
    if params.get("column", None) is not None:
        cargs.extend([
            "-column",
            params.get("column", None)
        ])
    if params.get("fraction", None) is not None:
        cargs.extend([
            "-placement",
            str(params.get("fraction", None))
        ])
    return cargs


def cifti_label_to_border_outputs(
    params: CiftiLabelToBorderParamsDict,
    execution: Execution,
) -> CiftiLabelToBorderOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelToBorderOutputs(
        root=execution.output_file("."),
        border=[cifti_label_to_border_border_outputs(i, execution) if cifti_label_to_border_border_outputs else None for i in params.get("border")] if params.get("border") else None,
    )
    return ret


def cifti_label_to_border_execute(
    params: CiftiLabelToBorderParamsDict,
    runner: Runner | None = None,
) -> CiftiLabelToBorderOutputs:
    """
    DRAW BORDERS AROUND CIFTI LABELS.
    
    For each surface, takes the labels on the matching structure and draws
    borders around the labels. Use -column to only draw borders around one label
    map.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelToBorderOutputs`).
    """
    cifti_label_to_border_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_LABEL_TO_BORDER_METADATA)
    params = execution.params(params)
    cargs = cifti_label_to_border_cargs(params, execution)
    ret = cifti_label_to_border_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_label_to_border(
    cifti_in: InputPathType,
    border: list[CiftiLabelToBorderBorderParamsDict] | None = None,
    column: str | None = None,
    fraction: float | None = None,
    runner: Runner | None = None,
) -> CiftiLabelToBorderOutputs:
    """
    DRAW BORDERS AROUND CIFTI LABELS.
    
    For each surface, takes the labels on the matching structure and draws
    borders around the labels. Use -column to only draw borders around one label
    map.
    
    Args:
        cifti_in: the input cifti dlabel file.
        border: specify output file for a surface structure.
        column: select a single column\
            \
            the column number or name.
        fraction: set how far along the edge border points are drawn\
            \
            fraction along edge from inside vertex (default 0.33).
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelToBorderOutputs`).
    """
    params = cifti_label_to_border_params(
        cifti_in=cifti_in,
        border=border,
        column=column,
        fraction=fraction,
    )
    return cifti_label_to_border_execute(params, runner)


__all__ = [
    "CIFTI_LABEL_TO_BORDER_METADATA",
    "CiftiLabelToBorderBorderOutputs",
    "CiftiLabelToBorderBorderParamsDict",
    "CiftiLabelToBorderBorderParamsDictTagged",
    "CiftiLabelToBorderOutputs",
    "CiftiLabelToBorderParamsDict",
    "CiftiLabelToBorderParamsDictTagged",
    "cifti_label_to_border",
    "cifti_label_to_border_border",
    "cifti_label_to_border_execute",
    "cifti_label_to_border_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

SURFACE_INFORMATION_METADATA = Metadata(
    id="87b1b891daeb7b0feeaa4cf02501f4785ccf44ce.workbench",
    name="surface-information",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_SurfaceInformationParamsDictNoTag = typing.TypedDict('_SurfaceInformationParamsDictNoTag', {
    "Surface File": InputPathType,
})
SurfaceInformationParamsDictTagged = typing.TypedDict('SurfaceInformationParamsDictTagged', {
    "@type": typing.Literal["workbench/surface-information"],
    "Surface File": InputPathType,
})
SurfaceInformationParamsDict = _SurfaceInformationParamsDictNoTag | SurfaceInformationParamsDictTagged


class SurfaceInformationOutputs(typing.NamedTuple):
    """
    Output object returned when calling `SurfaceInformationParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""


def surface_information_params(
    surface_file: InputPathType,
) -> SurfaceInformationParamsDictTagged:
    """
    Build parameters.
    
    Args:
        surface_file: Surface for which information is displayed.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/surface-information",
        "Surface File": surface_file,
    }
    return params


def surface_information_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `SurfaceInformationParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("Surface File", None) is None:
        raise StyxValidationError("`Surface File` must not be None")
    if not isinstance(params["Surface File"], (pathlib.Path, str)):
        raise StyxValidationError(f'`Surface File` has the wrong type: Received `{type(params.get("Surface File", None))}` expected `InputPathType`')


def surface_information_cargs(
    params: SurfaceInformationParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-surface-information"
    ])
    cargs.append(execution.input_file(params.get("Surface File", None)))
    return cargs


def surface_information_outputs(
    params: SurfaceInformationParamsDict,
    execution: Execution,
) -> SurfaceInformationOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = SurfaceInformationOutputs(
        root=execution.output_file("."),
    )
    return ret


def surface_information_execute(
    params: SurfaceInformationParamsDict,
    runner: Runner | None = None,
) -> SurfaceInformationOutputs:
    """
    DISPLAY INFORMATION ABOUT A SURFACE.
    
    Information about surface is displayed including vertices,
    triangles, bounding box, and spacing.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInformationOutputs`).
    """
    surface_information_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(SURFACE_INFORMATION_METADATA)
    params = execution.params(params)
    cargs = surface_information_cargs(params, execution)
    ret = surface_information_outputs(params, execution)
    execution.run(cargs)
    return ret


def surface_information(
    surface_file: InputPathType,
    runner: Runner | None = None,
) -> SurfaceInformationOutputs:
    """
    DISPLAY INFORMATION ABOUT A SURFACE.
    
    Information about surface is displayed including vertices,
    triangles, bounding box, and spacing.
    
    Args:
        surface_file: Surface for which information is displayed.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `SurfaceInformationOutputs`).
    """
    params = surface_information_params(
        surface_file=surface_file,
    )
    return surface_information_execute(params, runner)


__all__ = [
    "SURFACE_INFORMATION_METADATA",
    "SurfaceInformationOutputs",
    "SurfaceInformationParamsDict",
    "SurfaceInformationParamsDictTagged",
    "surface_information",
    "surface_information_execute",
    "surface_information_params",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_CHANGE_MAPPING_METADATA = Metadata(
    id="98052ea0462fd3e196a00f6d6e0bcbac9ddc17e4.workbench",
    name="cifti-change-mapping",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiChangeMappingSeriesParamsDictNoTag = typing.TypedDict('_CiftiChangeMappingSeriesParamsDictNoTag', {
    "step": float,
    "start": float,
    "unit": typing.NotRequired[str | None],
})
CiftiChangeMappingSeriesParamsDictTagged = typing.TypedDict('CiftiChangeMappingSeriesParamsDictTagged', {
    "@type": typing.Literal["series"],
    "step": float,
    "start": float,
    "unit": typing.NotRequired[str | None],
})
CiftiChangeMappingSeriesParamsDict = _CiftiChangeMappingSeriesParamsDictNoTag | CiftiChangeMappingSeriesParamsDictTagged


_CiftiChangeMappingScalarParamsDictNoTag = typing.TypedDict('_CiftiChangeMappingScalarParamsDictNoTag', {
    "file": typing.NotRequired[str | None],
})
CiftiChangeMappingScalarParamsDictTagged = typing.TypedDict('CiftiChangeMappingScalarParamsDictTagged', {
    "@type": typing.Literal["scalar"],
    "file": typing.NotRequired[str | None],
})
CiftiChangeMappingScalarParamsDict = _CiftiChangeMappingScalarParamsDictNoTag | CiftiChangeMappingScalarParamsDictTagged


_CiftiChangeMappingFromCiftiParamsDictNoTag = typing.TypedDict('_CiftiChangeMappingFromCiftiParamsDictNoTag', {
    "template-cifti": InputPathType,
    "direction": str,
})
CiftiChangeMappingFromCiftiParamsDictTagged = typing.TypedDict('CiftiChangeMappingFromCiftiParamsDictTagged', {
    "@type": typing.Literal["from-cifti"],
    "template-cifti": InputPathType,
    "direction": str,
})
CiftiChangeMappingFromCiftiParamsDict = _CiftiChangeMappingFromCiftiParamsDictNoTag | CiftiChangeMappingFromCiftiParamsDictTagged


_CiftiChangeMappingParamsDictNoTag = typing.TypedDict('_CiftiChangeMappingParamsDictNoTag', {
    "data-cifti": InputPathType,
    "direction": str,
    "cifti-out": str,
    "series": typing.NotRequired[CiftiChangeMappingSeriesParamsDict | None],
    "scalar": typing.NotRequired[CiftiChangeMappingScalarParamsDict | None],
    "from-cifti": typing.NotRequired[CiftiChangeMappingFromCiftiParamsDict | None],
})
CiftiChangeMappingParamsDictTagged = typing.TypedDict('CiftiChangeMappingParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-change-mapping"],
    "data-cifti": InputPathType,
    "direction": str,
    "cifti-out": str,
    "series": typing.NotRequired[CiftiChangeMappingSeriesParamsDict | None],
    "scalar": typing.NotRequired[CiftiChangeMappingScalarParamsDict | None],
    "from-cifti": typing.NotRequired[CiftiChangeMappingFromCiftiParamsDict | None],
})
CiftiChangeMappingParamsDict = _CiftiChangeMappingParamsDictNoTag | CiftiChangeMappingParamsDictTagged


def cifti_change_mapping_series(
    step: float,
    start: float,
    unit: str | None = None,
) -> CiftiChangeMappingSeriesParamsDictTagged:
    """
    Build parameters.
    
    Args:
        step: increment between series points.
        start: start value of the series.
        unit: select unit for series (default SECOND)\
            \
            unit identifier.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "series",
        "step": step,
        "start": start,
    }
    if unit is not None:
        params["unit"] = unit
    return params


def cifti_change_mapping_series_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiChangeMappingSeriesParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("step", None) is None:
        raise StyxValidationError("`step` must not be None")
    if not isinstance(params["step"], (float, int)):
        raise StyxValidationError(f'`step` has the wrong type: Received `{type(params.get("step", None))}` expected `float`')
    if params.get("start", None) is None:
        raise StyxValidationError("`start` must not be None")
    if not isinstance(params["start"], (float, int)):
        raise StyxValidationError(f'`start` has the wrong type: Received `{type(params.get("start", None))}` expected `float`')
    if params.get("unit", None) is not None:
        if not isinstance(params["unit"], str):
            raise StyxValidationError(f'`unit` has the wrong type: Received `{type(params.get("unit", None))}` expected `str | None`')


def cifti_change_mapping_series_cargs(
    params: CiftiChangeMappingSeriesParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-series",
        str(params.get("step", None)),
        str(params.get("start", None))
    ])
    if params.get("unit", None) is not None:
        cargs.extend([
            "-unit",
            params.get("unit", None)
        ])
    return cargs


def cifti_change_mapping_scalar(
    file: str | None = None,
) -> CiftiChangeMappingScalarParamsDictTagged:
    """
    Build parameters.
    
    Args:
        file: specify names for the maps\
            \
            text file containing map names, one per line.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "scalar",
    }
    if file is not None:
        params["file"] = file
    return params


def cifti_change_mapping_scalar_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiChangeMappingScalarParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("file", None) is not None:
        if not isinstance(params["file"], str):
            raise StyxValidationError(f'`file` has the wrong type: Received `{type(params.get("file", None))}` expected `str | None`')


def cifti_change_mapping_scalar_cargs(
    params: CiftiChangeMappingScalarParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.append("-scalar")
    if params.get("file", None) is not None:
        cargs.extend([
            "-name-file",
            params.get("file", None)
        ])
    return cargs


def cifti_change_mapping_from_cifti(
    template_cifti: InputPathType,
    direction: str,
) -> CiftiChangeMappingFromCiftiParamsDictTagged:
    """
    Build parameters.
    
    Args:
        template_cifti: a cifti file containing the desired mapping.
        direction: which direction to copy the mapping from.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "from-cifti",
        "template-cifti": template_cifti,
        "direction": direction,
    }
    return params


def cifti_change_mapping_from_cifti_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiChangeMappingFromCiftiParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("template-cifti", None) is None:
        raise StyxValidationError("`template-cifti` must not be None")
    if not isinstance(params["template-cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`template-cifti` has the wrong type: Received `{type(params.get("template-cifti", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')


def cifti_change_mapping_from_cifti_cargs(
    params: CiftiChangeMappingFromCiftiParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "-from-cifti",
        execution.input_file(params.get("template-cifti", None)),
        params.get("direction", None)
    ])
    return cargs


class CiftiChangeMappingOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiChangeMappingParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    cifti_out: OutputPathType
    """the output cifti file"""


def cifti_change_mapping_params(
    data_cifti: InputPathType,
    direction: str,
    cifti_out: str,
    series: CiftiChangeMappingSeriesParamsDict | None = None,
    scalar: CiftiChangeMappingScalarParamsDict | None = None,
    from_cifti: CiftiChangeMappingFromCiftiParamsDict | None = None,
) -> CiftiChangeMappingParamsDictTagged:
    """
    Build parameters.
    
    Args:
        data_cifti: the cifti file to use the data from.
        direction: which direction on <data-cifti> to replace the mapping.
        cifti_out: the output cifti file.
        series: set the mapping to series.
        scalar: set the mapping to scalar.
        from_cifti: copy mapping from another cifti file.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-change-mapping",
        "data-cifti": data_cifti,
        "direction": direction,
        "cifti-out": cifti_out,
    }
    if series is not None:
        params["series"] = series
    if scalar is not None:
        params["scalar"] = scalar
    if from_cifti is not None:
        params["from-cifti"] = from_cifti
    return params


def cifti_change_mapping_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiChangeMappingParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("data-cifti", None) is None:
        raise StyxValidationError("`data-cifti` must not be None")
    if not isinstance(params["data-cifti"], (pathlib.Path, str)):
        raise StyxValidationError(f'`data-cifti` has the wrong type: Received `{type(params.get("data-cifti", None))}` expected `InputPathType`')
    if params.get("direction", None) is None:
        raise StyxValidationError("`direction` must not be None")
    if not isinstance(params["direction"], str):
        raise StyxValidationError(f'`direction` has the wrong type: Received `{type(params.get("direction", None))}` expected `str`')
    if params.get("cifti-out", None) is None:
        raise StyxValidationError("`cifti-out` must not be None")
    if not isinstance(params["cifti-out"], str):
        raise StyxValidationError(f'`cifti-out` has the wrong type: Received `{type(params.get("cifti-out", None))}` expected `str`')
    if params.get("series", None) is not None:
        cifti_change_mapping_series_validate(params["series"])
    if params.get("scalar", None) is not None:
        cifti_change_mapping_scalar_validate(params["scalar"])
    if params.get("from-cifti", None) is not None:
        cifti_change_mapping_from_cifti_validate(params["from-cifti"])


def cifti_change_mapping_cargs(
    params: CiftiChangeMappingParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-change-mapping"
    ])
    cargs.append(execution.input_file(params.get("data-cifti", None)))
    cargs.append(params.get("direction", None))
    cargs.append(params.get("cifti-out", None))
    if params.get("series", None) is not None or params.get("scalar", None) is not None or params.get("from-cifti", None) is not None:
        cargs.extend([
            *(cifti_change_mapping_series_cargs(params.get("series", None), execution) if (params.get("series", None) is not None) else []),
            *(cifti_change_mapping_scalar_cargs(params.get("scalar", None), execution) if (params.get("scalar", None) is not None) else []),
            *(cifti_change_mapping_from_cifti_cargs(params.get("from-cifti", None), execution) if (params.get("from-cifti", None) is not None) else [])
        ])
    return cargs


def cifti_change_mapping_outputs(
    params: CiftiChangeMappingParamsDict,
    execution: Execution,
) -> CiftiChangeMappingOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiChangeMappingOutputs(
        root=execution.output_file("."),
        cifti_out=execution.output_file(params.get("cifti-out", None)),
    )
    return ret


def cifti_change_mapping_execute(
    params: CiftiChangeMappingParamsDict,
    runner: Runner | None = None,
) -> CiftiChangeMappingOutputs:
    """
    CONVERT TO SCALAR, COPY MAPPING, ETC.
    
    Take an existing cifti file and change one of the mappings. Exactly one of
    -series, -scalar, or -from-cifti must be specified. The direction can be
    either an integer starting from 1, or the strings 'ROW' or 'COLUMN'.
    
    The argument to -unit must be one of the following:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiChangeMappingOutputs`).
    """
    cifti_change_mapping_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_CHANGE_MAPPING_METADATA)
    params = execution.params(params)
    cargs = cifti_change_mapping_cargs(params, execution)
    ret = cifti_change_mapping_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_change_mapping(
    data_cifti: InputPathType,
    direction: str,
    cifti_out: str,
    series: CiftiChangeMappingSeriesParamsDict | None = None,
    scalar: CiftiChangeMappingScalarParamsDict | None = None,
    from_cifti: CiftiChangeMappingFromCiftiParamsDict | None = None,
    runner: Runner | None = None,
) -> CiftiChangeMappingOutputs:
    """
    CONVERT TO SCALAR, COPY MAPPING, ETC.
    
    Take an existing cifti file and change one of the mappings. Exactly one of
    -series, -scalar, or -from-cifti must be specified. The direction can be
    either an integer starting from 1, or the strings 'ROW' or 'COLUMN'.
    
    The argument to -unit must be one of the following:
    
    SECOND
    HERTZ
    METER
    RADIAN.
    
    Args:
        data_cifti: the cifti file to use the data from.
        direction: which direction on <data-cifti> to replace the mapping.
        cifti_out: the output cifti file.
        series: set the mapping to series.
        scalar: set the mapping to scalar.
        from_cifti: copy mapping from another cifti file.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiChangeMappingOutputs`).
    """
    params = cifti_change_mapping_params(
        data_cifti=data_cifti,
        direction=direction,
        cifti_out=cifti_out,
        series=series,
        scalar=scalar,
        from_cifti=from_cifti,
    )
    return cifti_change_mapping_execute(params, runner)


__all__ = [
    "CIFTI_CHANGE_MAPPING_METADATA",
    "CiftiChangeMappingFromCiftiParamsDict",
    "CiftiChangeMappingFromCiftiParamsDictTagged",
    "CiftiChangeMappingOutputs",
    "CiftiChangeMappingParamsDict",
    "CiftiChangeMappingParamsDictTagged",
    "CiftiChangeMappingScalarParamsDict",
    "CiftiChangeMappingScalarParamsDictTagged",
    "CiftiChangeMappingSeriesParamsDict",
    "CiftiChangeMappingSeriesParamsDictTagged",
    "cifti_change_mapping",
    "cifti_change_mapping_execute",
    "cifti_change_mapping_from_cifti",
    "cifti_change_mapping_params",
    "cifti_change_mapping_scalar",
    "cifti_change_mapping_series",
]



# This file was auto generated by Styx.
# Do not edit this file directly.

import typing
import pathlib
from styxdefs import *

CIFTI_LABEL_PROBABILITY_METADATA = Metadata(
    id="0435d93a94c261bb607b4770626109d9ebab1571.workbench",
    name="cifti-label-probability",
    package="workbench",
    container_image_tag="nx10x/workbench:2.1.0",
)


_CiftiLabelProbabilityParamsDictNoTag = typing.TypedDict('_CiftiLabelProbabilityParamsDictNoTag', {
    "label-maps": InputPathType,
    "probability-dscalar-out": str,
    "exclude-unlabeled": bool,
})
CiftiLabelProbabilityParamsDictTagged = typing.TypedDict('CiftiLabelProbabilityParamsDictTagged', {
    "@type": typing.Literal["workbench/cifti-label-probability"],
    "label-maps": InputPathType,
    "probability-dscalar-out": str,
    "exclude-unlabeled": bool,
})
CiftiLabelProbabilityParamsDict = _CiftiLabelProbabilityParamsDictNoTag | CiftiLabelProbabilityParamsDictTagged


class CiftiLabelProbabilityOutputs(typing.NamedTuple):
    """
    Output object returned when calling `CiftiLabelProbabilityParamsDict(...)`.
    """
    root: OutputPathType
    """Output root folder. This is the root folder for all outputs."""
    probability_dscalar_out: OutputPathType
    """the relative frequencies of each label at each vertex/voxel"""


def cifti_label_probability_params(
    label_maps: InputPathType,
    probability_dscalar_out: str,
    exclude_unlabeled: bool = False,
) -> CiftiLabelProbabilityParamsDictTagged:
    """
    Build parameters.
    
    Args:
        label_maps: cifti dlabel file containing individual label maps from\
            many subjects.
        probability_dscalar_out: the relative frequencies of each label at each\
            vertex/voxel.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
    Returns:
        Parameter dictionary
    """
    params = {
        "@type": "workbench/cifti-label-probability",
        "label-maps": label_maps,
        "probability-dscalar-out": probability_dscalar_out,
        "exclude-unlabeled": exclude_unlabeled,
    }
    return params


def cifti_label_probability_validate(
    params: typing.Any,
) -> None:
    """
    Validate parameters. Throws an error if `params` is not a valid
    `CiftiLabelProbabilityParamsDict` object.
    
    Args:
        params: The parameters object to validate.
    """
    if params is None or not isinstance(params, dict):
        raise StyxValidationError(f'Params object has the wrong type \'{type(params)}\'')
    if params.get("label-maps", None) is None:
        raise StyxValidationError("`label-maps` must not be None")
    if not isinstance(params["label-maps"], (pathlib.Path, str)):
        raise StyxValidationError(f'`label-maps` has the wrong type: Received `{type(params.get("label-maps", None))}` expected `InputPathType`')
    if params.get("probability-dscalar-out", None) is None:
        raise StyxValidationError("`probability-dscalar-out` must not be None")
    if not isinstance(params["probability-dscalar-out"], str):
        raise StyxValidationError(f'`probability-dscalar-out` has the wrong type: Received `{type(params.get("probability-dscalar-out", None))}` expected `str`')
    if params.get("exclude-unlabeled", False) is None:
        raise StyxValidationError("`exclude-unlabeled` must not be None")
    if not isinstance(params["exclude-unlabeled"], bool):
        raise StyxValidationError(f'`exclude-unlabeled` has the wrong type: Received `{type(params.get("exclude-unlabeled", False))}` expected `bool`')


def cifti_label_probability_cargs(
    params: CiftiLabelProbabilityParamsDict,
    execution: Execution,
) -> list[str]:
    """
    Build command-line arguments from parameters.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Command-line arguments.
    """
    cargs = []
    cargs.extend([
        "wb_command",
        "-cifti-label-probability"
    ])
    cargs.append(execution.input_file(params.get("label-maps", None)))
    cargs.append(params.get("probability-dscalar-out", None))
    if params.get("exclude-unlabeled", False):
        cargs.append("-exclude-unlabeled")
    return cargs


def cifti_label_probability_outputs(
    params: CiftiLabelProbabilityParamsDict,
    execution: Execution,
) -> CiftiLabelProbabilityOutputs:
    """
    Build outputs object containing output file paths and possibly stdout/stderr.
    
    Args:
        params: The parameters.
        execution: The execution object for resolving input paths.
    Returns:
        Outputs object.
    """
    ret = CiftiLabelProbabilityOutputs(
        root=execution.output_file("."),
        probability_dscalar_out=execution.output_file(params.get("probability-dscalar-out", None)),
    )
    return ret


def cifti_label_probability_execute(
    params: CiftiLabelProbabilityParamsDict,
    runner: Runner | None = None,
) -> CiftiLabelProbabilityOutputs:
    """
    FIND FREQUENCY OF CIFTI LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that
    vertex/voxel, divided by the number of input maps.
    
    Args:
        params: The parameters.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelProbabilityOutputs`).
    """
    cifti_label_probability_validate(params)
    runner = runner or get_global_runner()
    execution = runner.start_execution(CIFTI_LABEL_PROBABILITY_METADATA)
    params = execution.params(params)
    cargs = cifti_label_probability_cargs(params, execution)
    ret = cifti_label_probability_outputs(params, execution)
    execution.run(cargs)
    return ret


def cifti_label_probability(
    label_maps: InputPathType,
    probability_dscalar_out: str,
    exclude_unlabeled: bool = False,
    runner: Runner | None = None,
) -> CiftiLabelProbabilityOutputs:
    """
    FIND FREQUENCY OF CIFTI LABELS.
    
    This command outputs a set of soft ROIs, one for each label in the input,
    where the value is how many of the input maps had that label at that
    vertex/voxel, divided by the number of input maps.
    
    Args:
        label_maps: cifti dlabel file containing individual label maps from\
            many subjects.
        probability_dscalar_out: the relative frequencies of each label at each\
            vertex/voxel.
        exclude_unlabeled: don't make a probability map of the unlabeled key.
        runner: Command runner.
    Returns:
        NamedTuple of outputs (described in `CiftiLabelProbabilityOutputs`).
    """
    params = cifti_label_probability_params(
        label_maps=label_maps,
        probability_dscalar_out=probability_dscalar_out,
        exclude_unlabeled=exclude_unlabeled,
    )
    return cifti_label_probability_execute(params, runner)


__all__ = [
    "CIFTI_LABEL_PROBABILITY_METADATA",
    "CiftiLabelProbabilityOutputs",
    "CiftiLabelProbabilityParamsDict",
    "CiftiLabelProbabilityParamsDictTagged",
    "cifti_label_probability",
    "cifti_label_probability_execute",
    "cifti_label_probability_params",
]
